Package: tubern
Title: R Client for the YouTube Analytics and Reporting API
Version: 0.5.0
Authors@R: person("Gaurav", "Sood", email = "gsood07@gmail.com", role = c("aut", "cre"))
Author: Gaurav Sood [aut, cre]
Maintainer: Gaurav Sood <gsood07@gmail.com>
Description: Provides access to YouTube Analytics API v2 for retrieving 
    YouTube Analytics data including views, engagement metrics, demographics, and 
    revenue data. Supports OAuth 2.0 authentication and channel group management. 
    See <https://developers.google.com/youtube/analytics/> for API documentation.
URL: https://github.com/gojiplus/tubern
BugReports: https://github.com/gojiplus/tubern/issues
Depends: R (>= 4.1.0)
License: MIT + file LICENSE
Encoding: UTF-8
Language: en-US
Imports: httr, jsonlite, checkmate, rlang (>= 1.1.0)
VignetteBuilder: knitr
Suggests: knitr (>= 1.11), testthat, httptest, rmarkdown, xml2, lintr,
        covr, ggplot2, tibble
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-12 05:41:31 UTC; soodoku
Repository: CRAN
Date/Publication: 2026-04-12 16:00:02 UTC
Built: R 4.5.2; ; 2026-04-15 05:07:22 UTC; unix

# Generated by roxygen2: do not edit by hand

export(add_group_item)
export(add_groups)
export(check_api_quota)
export(delete_group)
export(delete_group_item)
export(diagnose_tubern)
export(get_audience_demographics)
export(get_available_dimensions)
export(get_available_metrics)
export(get_channel_overview)
export(get_common_date_ranges)
export(get_daily_performance)
export(get_geographic_performance)
export(get_report)
export(get_revenue_report)
export(get_top_videos)
export(list_group_items)
export(list_groups)
export(resolve_date_range)
export(update_group)
export(yt_export_csv)
export(yt_extract_summary)
export(yt_oauth)
export(yt_quick_plot)
export(yt_to_dataframe)
export(yt_to_tibble)
importFrom(checkmate,assert_character)
importFrom(checkmate,assert_flag)
importFrom(checkmate,assert_int)
importFrom(checkmate,assert_list)
importFrom(checkmate,assert_number)
importFrom(checkmate,assert_string)
importFrom(checkmate,check_string)
importFrom(graphics,barplot)
importFrom(graphics,points)
importFrom(httr,DELETE)
importFrom(httr,GET)
importFrom(httr,POST)
importFrom(httr,PUT)
importFrom(httr,authenticate)
importFrom(httr,config)
importFrom(httr,content)
importFrom(httr,oauth2.0_token)
importFrom(httr,oauth_app)
importFrom(httr,oauth_endpoints)
importFrom(httr,stop_for_status)
importFrom(httr,upload_file)
importFrom(jsonlite,toJSON)
importFrom(rlang,abort)
importFrom(rlang,caller_env)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(stats,median)
importFrom(utils,URLencode)
importFrom(utils,adist)
importFrom(utils,packageVersion)
importFrom(utils,write.csv)

## tubern: R Client for the YouTube Analytics and Reporting API

[![CRAN_Status_Badge](https://www.r-pkg.org:443/badges/version/tubern)](https://www.r-pkg.org:443/badges/version/tubern)
![](https://cranlogs.r-pkg.org/badges/grand-total/tubern)
[![R-CMD-check](https://github.com/gojiplus/tubern/actions/workflows/R-CMD-check.yaml/badge.svg)](https://github.com/gojiplus/tubern/actions)

tubern provides an R interface to the [YouTube Analytics API v2](https://developers.google.com/youtube/analytics), allowing you to retrieve YouTube Analytics data for channels and content owners. The package supports authentication via OAuth 2.0 and provides functions for getting analytics reports and managing channel groups.

## Features

- **🚀 Enhanced User Experience**: Simplified functions for common analytics tasks
- **📅 Smart Date Handling**: Use intuitive relative dates like "last_30_days", "this_month"
- **🔍 Intelligent Validation**: Get helpful suggestions for metrics, dimensions, and parameters
- **📊 Built-in Data Transformation**: Convert API responses to data.frames, tibbles, and CSV exports
- **📈 Quick Visualization**: Generate plots with one function call
- **🛠️ Advanced Error Handling**: Informative error messages and diagnostic tools
- **🔐 Streamlined Authentication**: Step-by-step OAuth setup with automatic scope detection
- **⚡ Pre-built Report Templates**: Channel overview, top videos, demographics, and more
- **🌍 Geographic & Demographic Analysis**: Built-in support for audience insights
- **💰 Revenue Analytics**: Comprehensive monetization reporting (with proper OAuth scope)

## Installation

### From CRAN
```r
install.packages("tubern")
```

### Development version from GitHub
```r
# install.packages("devtools")
devtools::install_github("gojiplus/tubern", build_vignettes = TRUE)
```

## Quick Start

### 1. Easy Authentication Setup

```r
library(tubern)

# First time? Get step-by-step setup guide
yt_oauth()

# With your credentials (basic analytics)
yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret")

# For revenue data (requires special permission)
yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret", scope = "monetary")
```

### 2. Get Analytics with Smart Dates

```r
# Use intuitive relative dates - no more manual date calculations!
overview <- get_channel_overview("last_30_days")
daily_trends <- get_daily_performance("this_month") 
top_videos <- get_top_videos("last_7_days")

# Or use traditional dates
report <- get_report(
  ids = "channel==MINE",
  metrics = "views,likes,comments",
  start_date = "2024-01-01",
  end_date = "2024-01-31"
)
```

### 3. Instant Data Analysis & Visualization

```r
# Convert to data.frame with clean column names
df <- yt_to_dataframe(daily_trends)
head(df)

# Quick visualization
yt_quick_plot(daily_trends)    # Auto-detects best chart type
yt_quick_plot(top_videos, chart_type = "bar")

# Export to CSV
yt_export_csv(overview, "my_channel_overview.csv")

# Get summary statistics
summary <- yt_extract_summary(overview)
```

### 4. Pre-built Analytics Templates

```r
# Channel performance overview
overview <- get_channel_overview("last_30_days")

# Top performing videos
top_videos <- get_top_videos("last_7_days", max_results = 20)

# Audience demographics
demographics <- get_audience_demographics("last_90_days")

# Geographic performance
geo_performance <- get_geographic_performance("this_month")

# Device/platform breakdown
device_performance <- get_report(
  ids = "channel==MINE",
  metrics = "views,estimatedMinutesWatched", 
  dimensions = "deviceType",
  start_date = "last_30_days"
)

# Revenue analysis (requires monetary scope)
revenue <- get_revenue_report("last_month")
```

### 5. Smart Validation & Help

```r
# Get available metrics and dimensions
get_available_metrics()
get_available_dimensions()

# Helpful error messages with suggestions
# get_report(ids = "channel==MINE", metrics = "vews", start_date = "last_7_days")
# → Error: Invalid metric(s): vews. Did you mean: 'vews' -> 'views'?

# Diagnostic tools
diagnose_tubern()  # Check setup
check_api_quota()  # Monitor API usage
```

## API Reference

### Core Functions

- `yt_oauth()`: Set up OAuth 2.0 authentication
- `get_report()`: Retrieve YouTube Analytics reports
- `list_groups()`: List channel groups
- `add_groups()`: Create new channel groups
- `update_group()`: Modify existing groups
- `delete_group()`: Remove groups
- `list_group_items()`: List items in a group
- `add_group_item()`: Add channels to groups
- `delete_group_item()`: Remove channels from groups

### Available Metrics

Common metrics you can request include:
- `views`: Number of times videos were viewed
- `likes`, `dislikes`: Engagement metrics
- `shares`: Number of times videos were shared
- `comments`: Number of comments
- `subscribersGained`, `subscribersLost`: Subscriber changes
- `estimatedMinutesWatched`: Total watch time in minutes
- `averageViewDuration`: Average time spent watching

For monetary metrics (requires monetary-analytics scope):
- `estimatedRevenue`: Estimated revenue
- `adEarnings`: Revenue from ads
- `impressionBasedCpm`: Cost per thousand impressions

### Available Dimensions

Dimensions allow you to segment your data:
- `video`: Individual video performance
- `day`: Daily breakdowns
- `country`: Geographic data
- `deviceType`: Desktop, mobile, tablet, etc.
- `operatingSystem`: OS breakdowns
- `ageGroup`, `gender`: Demographic data (limited availability)

## Authentication Setup

1. Go to the [Google Cloud Console](https://console.cloud.google.com/)
2. Create a new project or select an existing one
3. Enable the YouTube Analytics API
4. Create OAuth 2.0 credentials (Desktop application type)
5. Use the client ID and secret in `yt_oauth()`

For detailed authentication instructions, see the [YouTube Analytics API documentation](https://developers.google.com/youtube/analytics/registering_an_application).

## Important Notes

- **API Changes**: This package has been updated to use YouTube Analytics API v2. Some parameter names and behaviors may have changed from previous versions.
- **Rate Limits**: The YouTube Analytics API has usage quotas. See the [quota usage documentation](https://developers.google.com/youtube/analytics/quota) for details.
- **Data Freshness**: Analytics data typically has a delay of 24-72 hours.

## Related Resources

- [YouTube Analytics API Documentation](https://developers.google.com/youtube/analytics)
- [YouTube Data API R Client](https://github.com/gojiplus/tuber) (for video metadata, not analytics)
- [Google APIs R Client](https://github.com/r-lib/gargle) (for general Google API authentication)

## License

Released under the [MIT License](http://opensource.org/licenses/MIT).

## Contributing

Contributions are welcome! Please read the [Contributor Code of Conduct](https://www.contributor-covenant.org/version/1/0/0/) and submit issues or pull requests on GitHub.

## Support

For bug reports and feature requests, please use the [GitHub issue tracker](https://github.com/gojiplus/tubern/issues).

## ----include = FALSE----------------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----setup--------------------------------------------------------------------
# library(tubern)
# 
# # First-time setup - shows setup guide
# yt_oauth()
# 
# # Or with your credentials
# yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret")

## ----dates--------------------------------------------------------------------
# # Get common date range options
# get_common_date_ranges()
# 
# # Use relative dates in reports
# report <- get_report(
#   ids = "channel==MINE",
#   metrics = "views,likes",
#   start_date = "last_30_days"  # End date calculated automatically
# )
# 
# # More examples
# get_report(ids = "channel==MINE", metrics = "views", start_date = "this_month")
# get_report(ids = "channel==MINE", metrics = "views", start_date = "last_quarter")
# get_report(ids = "channel==MINE", metrics = "views", start_date = "yesterday")

## ----channel_overview---------------------------------------------------------
# # Get comprehensive channel performance
# overview <- get_channel_overview("last_30_days")
# 
# # Customize what metrics to include
# overview <- get_channel_overview(
#   "this_month",
#   include_engagement = TRUE,
#   include_subscribers = TRUE
# )

## ----top_videos---------------------------------------------------------------
# # Get top 10 videos by views
# top_videos <- get_top_videos("last_7_days")
# 
# # Get more videos with custom metrics
# top_videos <- get_top_videos(
#   "last_30_days",
#   max_results = 25,
#   metrics = c("views", "likes", "comments", "shares", "estimatedMinutesWatched")
# )

## ----demographics-------------------------------------------------------------
# # Full demographic breakdown
# demographics <- get_audience_demographics("last_90_days")
# 
# # Age groups only
# age_data <- get_audience_demographics("this_month", dimension = "ageGroup")
# 
# # Gender breakdown only
# gender_data <- get_audience_demographics("this_quarter", dimension = "gender")

## ----geographic---------------------------------------------------------------
# # Top countries by views
# geo_performance <- get_geographic_performance("last_30_days")
# 
# # US states/provinces (requires filtering to US)
# us_states <- get_geographic_performance(
#   "this_month",
#   dimension = "province",
#   filters = "country==US"
# )

## ----devices------------------------------------------------------------------
# # Performance by device type - use get_report directly
# device_performance <- get_report(
#   ids = "channel==MINE",
#   metrics = "views,estimatedMinutesWatched",
#   dimensions = "deviceType",
#   start_date = "last_30_days"
# )
# 
# # Operating system breakdown
# os_performance <- get_report(
#   ids = "channel==MINE",
#   metrics = "views,estimatedMinutesWatched",
#   dimensions = "operatingSystem",
#   start_date = "this_month"
# )
# 
# # Both device and OS
# device_os <- get_report(
#   ids = "channel==MINE",
#   metrics = "views,estimatedMinutesWatched",
#   dimensions = "deviceType,operatingSystem",
#   start_date = "last_7_days"
# )

## ----daily--------------------------------------------------------------------
# # Daily performance trends
# daily_data <- get_daily_performance("last_30_days")
# 
# # With engagement metrics
# daily_engagement <- get_daily_performance(
#   "this_month",
#   metrics = c("views", "likes", "comments", "shares")
# )

## ----revenue------------------------------------------------------------------
# # Set up OAuth with monetary scope first
# yt_oauth("your-client-id", "your-client-secret", scope = "monetary")
# 
# # Get revenue data
# revenue <- get_revenue_report("last_month")
# 
# # Revenue in different currency
# revenue_eur <- get_revenue_report("this_quarter", currency = "EUR")

## ----validation---------------------------------------------------------------
# # Invalid metrics get helpful suggestions
# # get_report(ids = "channel==MINE", metrics = "vews", start_date = "last_7_days")
# # Error: Invalid metric(s): vews
# # Did you mean: 'vews' -> 'views'
# 
# # Check available metrics and dimensions
# get_available_metrics()
# get_available_metrics("view")  # Filter by pattern
# 
# get_available_dimensions()
# get_available_dimensions("country|city")  # Geographic dimensions

## ----dataframes---------------------------------------------------------------
# # Get data and convert to clean data.frame
# report <- get_daily_performance("last_30_days")
# df <- yt_to_dataframe(report)
# head(df)
# 
# # Convert to tibble (if tibble package installed)
# tbl <- yt_to_tibble(report)
# 
# # Keep original column names
# df_orig <- yt_to_dataframe(report, clean_names = FALSE)

## ----export-------------------------------------------------------------------
# # Export to CSV
# report <- get_top_videos("last_7_days")
# file_path <- yt_export_csv(report, "top_videos.csv")
# 
# # Auto-generated filename with timestamp
# file_path <- yt_export_csv(report)

## ----plots--------------------------------------------------------------------
# # Automatic plot creation (requires ggplot2)
# daily_report <- get_daily_performance("last_30_days")
# yt_quick_plot(daily_report)  # Line plot over time
# 
# # Bar chart for top videos
# top_videos <- get_top_videos("last_7_days")
# yt_quick_plot(top_videos, chart_type = "bar")
# 
# # Custom x and y columns
# geo_data <- get_geographic_performance("last_30_days")
# yt_quick_plot(geo_data, x_col = "country", y_col = "views")

## ----summary------------------------------------------------------------------
# # Get summary statistics
# report <- get_channel_overview("last_30_days")
# summary <- yt_extract_summary(report)
# print(summary)

## ----diagnostics--------------------------------------------------------------
# # Check if everything is set up correctly
# diagnose_tubern()
# 
# # Check API quota status
# check_api_quota()

## ----errors-------------------------------------------------------------------
# # tubern now provides helpful error messages and suggestions
# # for authentication, API quota, parameter errors, etc.
# 
# # Example: If OAuth token expires, tubern will suggest running yt_oauth() again
# # Example: If API quota is exceeded, tubern will suggest ways to reduce usage

## ----workflow1----------------------------------------------------------------
# # Set up authentication
# yt_oauth("your-client-id", "your-client-secret")
# 
# # Get overview data
# overview <- get_channel_overview("last_30_days")
# overview_df <- yt_to_dataframe(overview)
# 
# # Get daily trends
# daily <- get_daily_performance("last_30_days")
# daily_df <- yt_to_dataframe(daily)
# 
# # Get top videos
# top_videos <- get_top_videos("last_30_days", max_results = 10)
# videos_df <- yt_to_dataframe(top_videos)
# 
# # Get geographic performance
# geo <- get_geographic_performance("last_30_days")
# geo_df <- yt_to_dataframe(geo)
# 
# # Create visualizations
# yt_quick_plot(daily)      # Daily trend
# yt_quick_plot(top_videos) # Top videos
# yt_quick_plot(geo)        # Geographic performance
# 
# # Export data
# yt_export_csv(overview, "channel_overview.csv")
# yt_export_csv(daily, "daily_performance.csv")
# yt_export_csv(top_videos, "top_videos.csv")

## ----workflow2----------------------------------------------------------------
# # Demographic analysis
# demographics <- get_audience_demographics("last_90_days")
# demo_df <- yt_to_dataframe(demographics)
# 
# # Device analysis
# devices <- get_report(
#   ids = "channel==MINE",
#   metrics = "views,estimatedMinutesWatched",
#   dimensions = "deviceType",
#   start_date = "last_90_days"
# )
# device_df <- yt_to_dataframe(devices)
# 
# # Geographic analysis
# geography <- get_geographic_performance("last_90_days", max_results = 50)
# geo_df <- yt_to_dataframe(geography)
# 
# # Combined analysis with dplyr (if available)
# if (require(dplyr)) {
#   # Top countries by watch time
#   top_countries <- geo_df %>%
#     arrange(desc(estimated_minutes_watched)) %>%
#     head(10)
# 
#   print(top_countries)
# }

## ----workflow3----------------------------------------------------------------
# # Analyze video performance over time
# videos_monthly <- get_top_videos("this_month", max_results = 50)
# videos_df <- yt_to_dataframe(videos_monthly)
# 
# # Compare with previous month
# videos_prev <- get_top_videos("last_month", max_results = 50)
# videos_prev_df <- yt_to_dataframe(videos_prev)
# 
# # Daily performance for trend analysis
# daily_trends <- get_daily_performance(
#   "last_30_days",
#   metrics = c("views", "estimatedMinutesWatched", "likes", "comments")
# )
# trends_df <- yt_to_dataframe(daily_trends)
# 
# # Visualize trends
# yt_quick_plot(daily_trends, x_col = "day", y_col = "views")

## -----------------------------------------------------------------------------
# # Run diagnostics
# diagnose_tubern()
# 
# # Re-authenticate
# yt_oauth("your-client-id", "your-client-secret")

## -----------------------------------------------------------------------------
# # Check quota status
# check_api_quota()
# 
# # Reduce data scope
# get_report(ids = "channel==MINE", metrics = "views",
#           start_date = "yesterday", max_results = 10)

## -----------------------------------------------------------------------------
# # Check available options
# get_available_metrics()
# get_available_dimensions()
# 
# # Use helper functions for validation
# get_channel_overview("last_7_days")  # Pre-validated parameters

#' List Groups
#'
#' @param filter Named Vector. Required. Only one of the two: id or mine.
#' id: Comma-separated list of YouTube group ID(s) to retrieve.
#' mine: Set to TRUE to retrieve all groups owned by the authenticated user.
#' @param page_token String. Optional. Identifies a specific page in the result set that should be returned.
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_GET}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groups/list}
#'
#' @examples
#' \dontrun{
#' list_groups(filter = c(mine = TRUE))
#' }

list_groups <- function (filter, page_token = NULL, ...) {
  if (!is.null(page_token)) {
    assert_string(page_token, .var.name = "page_token")
  }

  querylist <- as.list(filter)
  if (!is.null(page_token)) {
    querylist$pageToken <- page_token
  }

  tubern_GET("groups", querylist, ...)
}

context("Groups endpoints")

test_that("add_groups constructs correct API call", {
  resource_details <- list(
    etag = "etag123",
    snippet = list(title = "hello"),
    contentDetails = list(itemType = "youtube#channel")
  )
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- add_groups(resource_details)
  expect_equal(res$method, "POST")
  expect_equal(res$path, "groups")
  expect_null(res$query)
  expect_equal(res$body, jsonlite::toJSON(resource_details, auto_unbox = TRUE))
})

test_that("list_groups constructs correct API call", {
  filter <- c(mine = "True")
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- list_groups(filter = filter)
  expect_equal(res$method, "GET")
  expect_equal(res$path, "groups")
  expect_equal(res$query, as.list(filter))
  expect_null(res$body)
})

test_that("update_group constructs correct API call", {
  resource_details <- list(id = "grp123", snippet = list(title = "updated"))
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- update_group(resource_details)
  expect_equal(res$method, "PUT")
  expect_equal(res$path, "groups")
  expect_null(res$query)
  expect_equal(res$body, jsonlite::toJSON(resource_details, auto_unbox = TRUE))
})

test_that("delete_group constructs correct API call", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- delete_group(id = "grp123")
  expect_equal(res$method, "DELETE")
  expect_equal(res$path, "groups")
  expect_equal(res$query, list(id = "grp123"))
  expect_equal(res$body, "")
})

#' Test Helpers for tubern
#'
#' Helper functions and utilities for testing

#' Skip test if no OAuth token is available
#'
#' Use this for integration tests that require real API access.
#' Tests using this will be skipped on CRAN and CI without tokens.
#'
#' @return Invisibly returns NULL; skips the test if no token is available
#' @keywords internal
skip_if_no_token <- function() {
  token <- getOption("google_token")
  if (is.null(token)) {
    testthat::skip("No OAuth token available. Run yt_oauth() to test with real API.")
  }
}

#' Create a mock API response for testing
#'
#' @param rows List of rows (each row is a vector of values)
#' @param column_headers List of column header definitions
#' @param query Optional query metadata
#' @return A list mimicking a YouTube Analytics API response
#' @keywords internal
mock_api_response <- function(rows = NULL,
                              column_headers = NULL,
                              query = list()) {
  if (is.null(column_headers)) {
    column_headers <- list(
      list(name = "day", dataType = "STRING"),
      list(name = "views", dataType = "INTEGER")
    )
  }

  if (is.null(rows)) {
    rows <- list(
      c("2023-01-01", "100"),
      c("2023-01-02", "200")
    )
  }

  list(
    kind = "youtubeAnalytics#resultTable",
    columnHeaders = column_headers,
    rows = rows,
    query = query
  )
}

#' Create a stub for API request that returns a mock response
#'
#' @param response The mock response to return
#' @return A function that can replace .api_request for testing
#' @keywords internal
create_api_stub <- function(response) {
  function(method, path, query = NULL, body = NULL, ...) {
    response
  }
}

#' Temporarily replace .api_request with a stub for testing
#'
#' @param stub_fn The stub function to use
#' @param code The code to execute with the stub in place
#' @return The result of evaluating the code
#' @keywords internal
with_api_stub <- function(stub_fn, code) {
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub_fn, ns = "tubern")
  force(code)
}

context("Get Report")

test_that("get_report validates ids parameter format", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    mock_api_response()
  }
  with_api_stub(stub, {
    expect_error(
      get_report(ids = "invalid", metrics = "views",
                 start_date = "2023-01-01", end_date = "2023-01-31"),
      class = "tubern_parameter_error"
    )
  })
})

test_that("get_report validates metrics parameter", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    mock_api_response()
  }
  with_api_stub(stub, {
    expect_error(
      get_report(ids = "channel==MINE", metrics = "invalid_metric",
                 start_date = "2023-01-01", end_date = "2023-01-31"),
      class = "tubern_parameter_error"
    )
  })
})

test_that("get_report requires start_date", {
  expect_error(
    get_report(ids = "channel==MINE", metrics = "views"),
    class = "tubern_parameter_error"
  )
})

test_that("get_report validates date format", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    mock_api_response()
  }
  with_api_stub(stub, {
    expect_error(
      get_report(ids = "channel==MINE", metrics = "views",
                 start_date = "2023/01/01", end_date = "2023-01-31"),
      class = "tubern_parameter_error"
    )
  })
})

test_that("get_report constructs correct query with valid parameters", {
  captured_query <- NULL
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    captured_query <<- query
    mock_api_response()
  }
  with_api_stub(stub, {
    get_report(ids = "channel==MINE", metrics = "views",
               start_date = "2023-01-01", end_date = "2023-01-31")
    expect_equal(captured_query$metrics, "views")
    expect_equal(captured_query$startDate, "2023-01-01")
    expect_equal(captured_query$endDate, "2023-01-31")
  })
})

test_that("get_report handles multiple metrics", {
  captured_query <- NULL
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    captured_query <<- query
    mock_api_response()
  }
  with_api_stub(stub, {
    get_report(ids = "channel==MINE", metrics = "views,likes,comments",
               start_date = "2023-01-01", end_date = "2023-01-31")
    expect_equal(captured_query$metrics, "views,likes,comments")
  })
})

test_that("get_report validates max_results bounds", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    mock_api_response()
  }
  with_api_stub(stub, {
    expect_error(
      get_report(ids = "channel==MINE", metrics = "views",
                 start_date = "2023-01-01", end_date = "2023-01-31",
                 max_results = 300),
      "max_results"
    )
  })
})

test_that("get_report with real API works", {
  skip_on_cran()
  skip_if_no_token()

  get_info <- get_report(ids = "channel==MINE", metrics = "views",
                         start_date = "2010-04-01", end_date = "2017-04-01")
  expect_type(get_info, "list")
})

#' YouTube Analytics API Validation Helpers
#'
#' Internal validation functions for metrics, dimensions, and parameters
#' using checkmate for robust parameter validation.
#' @name validation_helpers
#' @importFrom checkmate assert_character assert_string assert_flag assert_int assert_number assert_list check_string
NULL

# Valid metrics for YouTube Analytics API v2
.valid_metrics <- list(
  # View metrics
  views = "Number of times videos were viewed",
  redViews = "Number of times videos were viewed by YouTube Premium members",
  engagedViews = "Number of views that went past the initial seconds",
  viewerPercentage = "Percentage of viewers who were logged in when watching",

  # Impression metrics
  videoThumbnailImpressions = "Number of times video thumbnails were displayed",
  videoThumbnailImpressionsClickRate = "Click-through rate for video thumbnails",

  # Engagement metrics
  likes = "Number of likes",
  dislikes = "Number of dislikes",
  comments = "Number of comments",
  shares = "Number of times videos were shared",
  subscribersGained = "Number of subscribers gained",
  subscribersLost = "Number of subscribers lost",
  videosAddedToPlaylists = "Number of times videos were added to playlists",
  videosRemovedFromPlaylists = "Number of times videos were removed from playlists",

  # Watch time metrics
  estimatedMinutesWatched = "Total minutes watched",
  estimatedRedMinutesWatched = "Minutes watched by YouTube Premium members",
  averageViewDuration = "Average duration of video views in seconds",
  averageViewPercentage = "Average percentage of video watched",

  # Playlist metrics
  playlistViews = "Number of times videos were viewed within playlist context",
  playlistStarts = "Number of times playlist playback was initiated",
  playlistSaves = "Number of times users saved the playlist",
  viewsPerPlaylistStart = "Average number of video views per playlist start",
  averageTimeInPlaylist = "Average time in seconds viewers spent in playlist",
  playlistAverageViewDuration = "Average duration of playlist video views",
  playlistEstimatedMinutesWatched = "Total minutes watched in playlist context",

  # Revenue metrics (requires monetary scope)
  estimatedRevenue = "Estimated total revenue",
  estimatedAdRevenue = "Estimated revenue from ads",
  estimatedRedPartnerRevenue = "Estimated revenue from YouTube Premium subscriptions",
  monetizedPlaybacks = "Number of monetized playbacks",
  playbackBasedCpm = "Cost per mille based on playbacks",
  cpm = "Cost per mille based on impressions",
  grossRevenue = "Gross revenue before revenue sharing",

  # Ad performance metrics
  adImpressions = "Number of ad impressions",

  # Card metrics
  cardImpressions = "Number of card impressions",
  cardClicks = "Number of card clicks",
  cardClickRate = "Click-through rate for cards",
  cardTeaserImpressions = "Number of card teaser impressions",
  cardTeaserClicks = "Number of card teaser clicks",
  cardTeaserClickRate = "Click-through rate for card teasers",

  # Annotation metrics
  annotationImpressions = "Number of annotation impressions",
  annotationClickableImpressions = "Number of clickable annotation impressions",
  annotationClicks = "Number of annotation clicks",
  annotationClickThroughRate = "Click-through rate for annotations",
  annotationClosableImpressions = "Number of closable annotation impressions",
  annotationCloses = "Number of annotation closes",
  annotationCloseRate = "Rate at which annotations were closed",

  # Livestream metrics
  averageConcurrentViewers = "Average number of concurrent viewers during live stream",
  peakConcurrentViewers = "Peak number of concurrent viewers during live stream",

  # Audience retention metrics
  audienceWatchRatio = "Ratio of views at each moment compared to initial views",
  relativeRetentionPerformance = "Retention performance relative to similar videos",
  startedWatching = "Number of views that started at this point",
  stoppedWatching = "Number of views that stopped at this point",
  totalSegmentImpressions = "Total number of impressions for this segment"
)

# Valid dimensions for YouTube Analytics API v2
.valid_dimensions <- list(
  # Core dimensions
  channel = "Channel identifier",
  video = "Video identifier",
  playlist = "Playlist identifier",

  # Time dimensions
  day = "Daily aggregation",
  month = "Monthly aggregation",

  # Geographic dimensions
  country = "Two-letter country code",
  province = "Province/state (US only)",
  city = "City",

  # Demographic dimensions
  ageGroup = "Age group of viewers",
  gender = "Gender of viewers",

  # Device dimensions
  deviceType = "Type of device used",
  operatingSystem = "Operating system",

  # Traffic source dimensions
  insightTrafficSourceType = "Traffic source type",
  insightTrafficSourceDetail = "Detailed traffic source",

  # Content dimensions
  creatorContentType = "Type of creator content",
  liveOrOnDemand = "Live or on-demand content",
  youtubeProduct = "YouTube product",

  # Playback dimensions
  insightPlaybackLocationType = "Playback location type",

  # Sharing dimensions
  sharingService = "Service used for sharing"
)

# Filter-only dimensions
.filter_only_dimensions <- c("continent", "subContinent", "group")

# Dimensions that require specific filters
.dimension_requirements <- list(
  province = "country==US"
)

#' Find close matches for a string
#'
#' @param x String to match
#' @param choices Valid choices to match against
#' @param max_dist Maximum edit distance for suggestions
#' @return Character vector of suggestions
#' @keywords internal
#' @noRd
.find_suggestions <- function(x, choices, max_dist = 3) {
  distances <- adist(x, choices)
  close_matches <- choices[distances <= max_dist]
  if (length(close_matches) > 0) {
    paste0("'", x, "' -> '", close_matches[which.min(distances[distances <= max_dist])], "'")
  } else {
    character(0)
  }
}

#' Validate metrics parameter
#'
#' @param metrics Character vector or comma-separated string of metrics
#' @return Character vector of validated metrics
#' @keywords internal
#' @noRd
.validate_metrics <- function(metrics) {
  if (is.null(metrics) || length(metrics) == 0) {
    tubern_abort(
      "At least one metric must be specified",
      class = "parameter"
    )
  }

  assert_character(metrics, min.len = 1, .var.name = "metrics")

  if (length(metrics) == 1 && grepl(",", metrics)) {
    metrics <- trimws(strsplit(metrics, ",")[[1]])
  }

  invalid_metrics <- metrics[!metrics %in% names(.valid_metrics)]
  if (length(invalid_metrics) > 0) {
    suggestions <- unlist(lapply(invalid_metrics, .find_suggestions, names(.valid_metrics)))

    msg <- paste0("Invalid metric(s): ", paste(invalid_metrics, collapse = ", "))
    if (length(suggestions) > 0) {
      msg <- paste0(msg, "\n\nDid you mean:\n", paste(suggestions, collapse = "\n"))
    }
    msg <- paste0(msg, "\n\nUse get_available_metrics() to see all valid metrics.")

    tubern_abort(msg, class = "parameter", invalid_metrics = invalid_metrics)
  }

  metrics
}

#' Validate dimensions parameter
#'
#' @param dimensions Character vector or comma-separated string of dimensions
#' @param filters Character string of filters (to check requirements)
#' @return Character vector of validated dimensions
#' @keywords internal
#' @noRd
.validate_dimensions <- function(dimensions, filters = NULL) {
  if (is.null(dimensions)) return(NULL)

  if (length(dimensions) == 1 && grepl(",", dimensions)) {
    dimensions <- trimws(strsplit(dimensions, ",")[[1]])
  }

  assert_character(dimensions, min.len = 1, .var.name = "dimensions")

  all_valid <- c(names(.valid_dimensions), .filter_only_dimensions)
  invalid_dims <- dimensions[!dimensions %in% all_valid]

  if (length(invalid_dims) > 0) {
    suggestions <- unlist(lapply(invalid_dims, .find_suggestions, all_valid))

    msg <- paste0("Invalid dimension(s): ", paste(invalid_dims, collapse = ", "))
    if (length(suggestions) > 0) {
      msg <- paste0(msg, "\n\nDid you mean:\n", paste(suggestions, collapse = "\n"))
    }
    msg <- paste0(msg, "\n\nUse get_available_dimensions() to see all valid dimensions.")

    tubern_abort(msg, class = "parameter", invalid_dimensions = invalid_dims)
  }

  filter_only_used <- dimensions[dimensions %in% .filter_only_dimensions]
  if (length(filter_only_used) > 0) {
    tubern_abort(
      paste0(
        "The following dimensions can only be used as filters: ",
        paste(filter_only_used, collapse = ", ")
      ),
      class = "parameter",
      filter_only_dimensions = filter_only_used
    )
  }

  for (dim in dimensions) {
    if (dim %in% names(.dimension_requirements)) {
      required_filter <- .dimension_requirements[[dim]]
      if (is.null(filters) || !grepl(required_filter, filters)) {
        tubern_abort(
          paste0("Dimension '", dim, "' requires filter: ", required_filter),
          class = "parameter",
          dimension = dim,
          required_filter = required_filter
        )
      }
    }
  }

  dimensions
}

#' Validate date parameters
#'
#' @param start_date Character string in YYYY-MM-DD format
#' @param end_date Character string in YYYY-MM-DD format
#' @return List with validated start_date and end_date
#' @keywords internal
#' @noRd
.validate_dates <- function(start_date, end_date) {
  if (is.null(start_date) || is.null(end_date)) {
    tubern_abort(
      "Both start_date and end_date are required",
      class = "parameter"
    )
  }

  assert_string(start_date, .var.name = "start_date")
  assert_string(end_date, .var.name = "end_date")

  date_pattern <- "^\\d{4}-\\d{2}-\\d{2}$"
  if (!grepl(date_pattern, start_date)) {
    tubern_abort(
      paste0("start_date must be in YYYY-MM-DD format, got: '", start_date, "'"),
      class = "parameter"
    )
  }
  if (!grepl(date_pattern, end_date)) {
    tubern_abort(
      paste0("end_date must be in YYYY-MM-DD format, got: '", end_date, "'"),
      class = "parameter"
    )
  }

  start_parsed <- tryCatch(as.Date(start_date), error = function(e) NULL)
  end_parsed <- tryCatch(as.Date(end_date), error = function(e) NULL)

  if (is.null(start_parsed)) {
    tubern_abort(
      paste0("Invalid start_date: '", start_date, "'"),
      class = "parameter"
    )
  }
  if (is.null(end_parsed)) {
    tubern_abort(
      paste0("Invalid end_date: '", end_date, "'"),
      class = "parameter"
    )
  }

  if (start_parsed > end_parsed) {
    tubern_abort(
      "start_date must be before or equal to end_date",
      class = "parameter",
      start_date = start_date,
      end_date = end_date
    )
  }

  if (end_parsed > Sys.Date()) {
    tubern_warn(
      "end_date is in the future, YouTube Analytics data may not be available",
      class = "parameter"
    )
  }

  list(start_date = start_date, end_date = end_date)
}

#' Validate filters parameter
#'
#' @param filters Character string of filters
#' @param dimensions Character vector of dimensions (for validation)
#' @return Character string of validated filters
#' @keywords internal
#' @noRd
.validate_filters <- function(filters, dimensions = NULL) {
  if (is.null(filters)) return(NULL)

  assert_string(filters, .var.name = "filters")

  if (!grepl("==", filters)) {
    tubern_abort(
      "Filters must be in format 'dimension==value' or 'dimension==value1,value2'",
      class = "parameter"
    )
  }

  filter_parts <- strsplit(filters, ";")[[1]]
  filter_dims <- vapply(filter_parts, function(x) {
    trimws(strsplit(x, "==")[[1]][1])
  }, character(1))

  all_valid <- c(names(.valid_dimensions), .filter_only_dimensions)
  invalid_filter_dims <- filter_dims[!filter_dims %in% all_valid]

  if (length(invalid_filter_dims) > 0) {
    tubern_abort(
      paste0("Invalid filter dimension(s): ", paste(invalid_filter_dims, collapse = ", ")),
      class = "parameter",
      invalid_dimensions = invalid_filter_dims
    )
  }

  if ("video" %in% filter_dims && !is.null(dimensions) && !"video" %in% dimensions) {
    tubern_inform(
      "When filtering by video, consider adding 'video' to dimensions to see individual video results"
    )
  }

  filters
}

#' Get available metrics with descriptions
#'
#' @param pattern Optional regex pattern to filter metrics
#' @return Named character vector of metrics and descriptions
#' @export
#' @examples
#' # Get all metrics
#' get_available_metrics()
#'
#' # Get only view-related metrics
#' get_available_metrics("view")
get_available_metrics <- function(pattern = NULL) {
  metrics <- .valid_metrics
  if (!is.null(pattern)) {
    assert_string(pattern, .var.name = "pattern")
    matches <- grepl(pattern, names(metrics), ignore.case = TRUE) |
               grepl(pattern, unlist(metrics), ignore.case = TRUE)
    metrics <- metrics[matches]
  }
  unlist(metrics)
}

#' Get available dimensions with descriptions
#'
#' @param pattern Optional regex pattern to filter dimensions
#' @return Named character vector of dimensions and descriptions
#' @export
#' @examples
#' # Get all dimensions
#' get_available_dimensions()
#'
#' # Get only geographic dimensions
#' get_available_dimensions("country|city")
get_available_dimensions <- function(pattern = NULL) {
  dimensions <- .valid_dimensions
  if (!is.null(pattern)) {
    assert_string(pattern, .var.name = "pattern")
    matches <- grepl(pattern, names(dimensions), ignore.case = TRUE) |
               grepl(pattern, unlist(dimensions), ignore.case = TRUE)
    dimensions <- dimensions[matches]
  }
  unlist(dimensions)
}

context("Validation helpers")

test_that(".validate_metrics rejects empty metrics", {
  expect_error(
    tubern:::.validate_metrics(NULL),
    class = "tubern_parameter_error"
  )
  expect_error(
    tubern:::.validate_metrics(character(0)),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_metrics accepts valid metrics", {
  result <- tubern:::.validate_metrics("views")
  expect_equal(result, "views")

  result <- tubern:::.validate_metrics(c("views", "likes"))
  expect_equal(result, c("views", "likes"))
})

test_that(".validate_metrics handles comma-separated string", {
  result <- tubern:::.validate_metrics("views,likes,comments")
  expect_equal(result, c("views", "likes", "comments"))
})

test_that(".validate_metrics rejects invalid metrics with suggestions", {
  err <- tryCatch(
    tubern:::.validate_metrics("viewz"),
    tubern_parameter_error = function(e) e
  )
  expect_true(grepl("Invalid metric", conditionMessage(err)))
  expect_true(grepl("Did you mean", conditionMessage(err)))
})

test_that(".validate_dimensions accepts valid dimensions", {
  result <- tubern:::.validate_dimensions("day")
  expect_equal(result, "day")

  result <- tubern:::.validate_dimensions(c("day", "country"))
  expect_equal(result, c("day", "country"))
})

test_that(".validate_dimensions handles NULL", {
  result <- tubern:::.validate_dimensions(NULL)
  expect_null(result)
})

test_that(".validate_dimensions rejects filter-only dimensions", {
  expect_error(
    tubern:::.validate_dimensions("continent"),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_dimensions enforces requirements", {
  expect_error(
    tubern:::.validate_dimensions("province", filters = NULL),
    class = "tubern_parameter_error"
  )

  result <- tubern:::.validate_dimensions("province", filters = "country==US")
  expect_equal(result, "province")
})

test_that(".validate_dates requires both dates", {
  expect_error(
    tubern:::.validate_dates(NULL, "2023-01-31"),
    class = "tubern_parameter_error"
  )
  expect_error(
    tubern:::.validate_dates("2023-01-01", NULL),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_dates validates format", {
  expect_error(
    tubern:::.validate_dates("2023/01/01", "2023-01-31"),
    class = "tubern_parameter_error"
  )
  expect_error(
    tubern:::.validate_dates("2023-01-01", "Jan 31, 2023"),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_dates validates date order", {
  expect_error(
    tubern:::.validate_dates("2023-01-31", "2023-01-01"),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_dates accepts valid dates", {
  result <- tubern:::.validate_dates("2023-01-01", "2023-01-31")
  expect_equal(result$start_date, "2023-01-01")
  expect_equal(result$end_date, "2023-01-31")
})

test_that(".validate_filters requires == syntax", {
  expect_error(
    tubern:::.validate_filters("country=US"),
    class = "tubern_parameter_error"
  )
})

test_that(".validate_filters accepts valid filters", {
  result <- tubern:::.validate_filters("country==US")
  expect_equal(result, "country==US")

  result <- tubern:::.validate_filters("country==US;video==abc123")
  expect_equal(result, "country==US;video==abc123")
})

test_that(".validate_filters handles NULL", {
  result <- tubern:::.validate_filters(NULL)
  expect_null(result)
})

test_that("get_available_metrics returns valid metrics", {
  metrics <- get_available_metrics()
  expect_type(metrics, "character")
  expect_true(length(metrics) > 0)
  expect_true("views" %in% names(metrics))
})

test_that("get_available_metrics filters by pattern", {
  metrics <- get_available_metrics("view")
  expect_true(all(grepl("view", names(metrics), ignore.case = TRUE) |
                  grepl("view", metrics, ignore.case = TRUE)))
})

test_that("get_available_dimensions returns valid dimensions", {
  dimensions <- get_available_dimensions()
  expect_type(dimensions, "character")
  expect_true(length(dimensions) > 0)
  expect_true("day" %in% names(dimensions))
})

#' Get Reports
#'
#' Retrieves YouTube Analytics reports containing YouTube Analytics data.
#'
#' @param ids String. Channel or content owner identifier.
#' For channels: \code{"channel==MINE"} (for authenticated user's channel) or
#' \code{"channel==UCxxxxxxxxxxxxxxx"} (for specific channel ID).
#' For content owners: \code{"contentOwner==ownerName"}.
#'
#' @param metrics String. Comma-separated list of YouTube Analytics metrics,
#' such as \code{views} or \code{likes,dislikes}.
#' @param start_date String. Start date for the report.
#' Can be in YYYY-MM-DD format or relative date like "last_30_days", "this_month", "yesterday".
#' @param end_date String. End date for the report.
#' Can be in YYYY-MM-DD format or relative date. If NULL and start_date is relative, will be calculated automatically.
#' @param currency Optional. String. Default is USD. Specifies what earnings
#' metrics like \code{earnings, adEarnings, grossRevenue, playbackBasedCpm,
#' impressionBasedCpm} will be reported in.
#' @param dimensions String. Optional. Comma-separated list of YouTube Analytics
#' dimensions, such as \code{video} or \code{ageGroup,gender}.
#' @param filters String. Optional. Dimension value filters. Multiple filters
#' can be separated by semicolons. For video filtering:
#' \code{"video==videoId1,videoId2"} (comma-separated video IDs).
#' For country filtering: \code{"country==US"}.
#' Combined example: \code{"video==videoId1,videoId2;country==US"}.
#' Note: When filtering by video IDs, ensure the \code{dimensions} parameter
#' includes "video".
#' @param include_historical_channel_data Boolean. Default is FALSE.
#' ``Whether the API response should include channels' watch time and view data
#' from the time period prior to when the channels were linked to the content
#' owner.''
#' @param max_results Integer. Optional. The maximum number of rows to
#' include in the response.
#' @param sort String. Optional A comma-separated list of dimensions or metrics
#' that determine the sort order for YouTube
#' @param start_index Integer. Optional. ``The 1-based index of the first
#' entity to retrieve.''
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_GET}}.
#'
#' @return named list
#'
#' @export
#'
#' @references
#' \url{https://developers.google.com/youtube/analytics/reference/reports/query}
#'
#' @section Troubleshooting 404 Errors:
#' If you encounter a 404 "Not Found" error, check the following:
#' \itemize{
#'   \item Ensure YouTube Analytics API is enabled in your Google Cloud Console
#'   project
#'   \item Verify your authentication token is valid and has the correct scopes
#'   \item Check that the channel ID (if using specific channel ID) exists and
#'   you have access to it
#'   \item Use \code{"channel==MINE"} to access your own authenticated channel's
#'   data
#' }
#'
#' @examples
#' \dontrun{
#' # Basic channel report
#' get_report(ids = "channel==MINE", metrics = "views",
#'            start_date = "2020-01-01", end_date = "2020-01-31")
#'
#' # Report with video filtering (requires dimensions = "video")
#' get_report(ids = "channel==MINE",
#'            metrics = "views,likes,comments",
#'            dimensions = "video",
#'            filters = "video==videoId1,videoId2",
#'            start_date = "2020-01-01", end_date = "2020-01-31")
#'
#' # Report with country filtering
#' get_report(ids = "channel==MINE",
#'            metrics = "views",
#'            filters = "country==US",
#'            start_date = "2020-01-01", end_date = "2020-01-31")
#' }

get_report <- function(ids, metrics, start_date = NULL, end_date = NULL,
                        currency = NULL, dimensions = NULL, filters = NULL,
                        include_historical_channel_data = NULL,
                        max_results = NULL, sort = NULL,
                        start_index = NULL, ...) {

  assert_string(ids, .var.name = "ids")

  if (!grepl("^(channel|contentOwner)==", ids)) {
    tubern_abort(
      c(
        "ids parameter must be in format:",
        "'channel==MINE', 'channel==CHANNEL_ID', or 'contentOwner==OWNER_NAME'"
      ),
      class = "parameter"
    )
  }

  if (is.null(start_date)) {
    tubern_abort("start_date is required", class = "parameter")
  }

  dates <- resolve_date_range(start_date, end_date)
  start_date <- dates$start_date
  end_date <- dates$end_date

  final_dates <- .validate_dates(start_date, end_date)
  start_date <- final_dates$start_date
  end_date <- final_dates$end_date

  metrics <- .validate_metrics(metrics)
  metrics_param <- paste(metrics, collapse = ",")

  dimensions <- .validate_dimensions(dimensions, filters)
  dimensions_param <- if (!is.null(dimensions)) paste(dimensions, collapse = ",") else NULL

  filters <- .validate_filters(filters, dimensions)

  if (!is.null(currency)) {
    assert_string(currency, .var.name = "currency")
    valid_currencies <- c("USD", "EUR", "GBP", "JPY", "KRW", "INR", "CAD", "AUD")
    if (!currency %in% valid_currencies) {
      tubern_warn(
        paste0("Currency '", currency, "' may not be supported. Common currencies: ",
               paste(valid_currencies, collapse = ", ")),
        class = "parameter"
      )
    }
  }

  if (!is.null(max_results)) {
    assert_int(max_results, lower = 1, upper = 200, .var.name = "max_results")
  }

  if (!is.null(start_index)) {
    assert_int(start_index, lower = 1, .var.name = "start_index")
  }

  querylist <- list(ids = URLencode(ids),
                    startDate = start_date,
                    endDate = end_date,
                    metrics = metrics_param,
                    currency = currency,
                    dimensions = dimensions_param,
                    filters = filters,
                    maxResults = max_results,
                    sort = sort,
                    includeHistoricalChannelData =
                      include_historical_channel_data,
                    startIndex = start_index)

  tubern_GET("reports", querylist, ...)
}

context("Reports endpoint")

## Test get_report against recorded fixture

test_that("get_report returns parsed list from fixture", {
  skip_on_cran()
  # Skip if no fixture is available
  fixture <- file.path(
    "tests", "testthat", "__httptest__", "GET-reports.json"
  )
  if (!file.exists(fixture)) {
    skip("No captured fixture for get_report. Run recording script to generate it.")
  }
  httptest::with_mock_api({
    res <- get_report(
      ids = "channel==MINE",
      metrics = "views",
      start_date = "2021-01-01",
      end_date = "2021-01-02"
    )
    expect_type(res, "list")
  })
})

context("Recording API fixtures")

## Manual script to record API fixture for get_report

test_that("record get_report fixture", {
  skip_on_cran()
  # Requires an OAuth token in options(google_token)
  if (is.null(getOption("google_token"))) {
    skip("No OAuth token available to record fixtures")
  }
  httptest::capture_requests({
    # call the real API; requires valid OAuth token
    get_report(
      ids = "channel==MINE",
      metrics = "views",
      start_date = "2021-01-01",
      end_date = "2021-01-02"
    )
  })
})

#' Common Report Helper Functions
#'
#' Pre-configured functions for common YouTube Analytics report types
#' @name report_helpers
NULL

#' Get channel overview report
#' 
#' Retrieves key channel performance metrics for a specified time period.
#' 
#' @param ids Channel identifier (default: "channel==MINE")
#' @param date_range Date range string like "last_30_days" or start date for custom range
#' @param end_date End date (only needed if date_range is a specific start date)
#' @param include_engagement Logical. Include likes, dislikes, comments, shares (default: TRUE)
#' @param include_subscribers Logical. Include subscriber metrics (default: TRUE)
#' @param ... Additional arguments passed to get_report()
#' @return API response with channel overview data
#' @export
#' @examples
#' \dontrun{
#' # Channel performance for last 30 days
#' get_channel_overview("last_30_days")
#' 
#' # Channel performance for specific date range
#' get_channel_overview("2024-01-01", end_date = "2024-01-31")
#' 
#' # Basic metrics only
#' get_channel_overview("this_month", include_engagement = FALSE, include_subscribers = FALSE)
#' }
get_channel_overview <- function(date_range, end_date = NULL, ids = "channel==MINE", 
                                include_engagement = TRUE, include_subscribers = TRUE, ...) {
  
  # Build metrics based on options
  metrics <- c("views", "estimatedMinutesWatched")
  
  if (include_engagement) {
    metrics <- c(metrics, "likes", "dislikes", "comments", "shares")
  }
  
  if (include_subscribers) {
    metrics <- c(metrics, "subscribersGained", "subscribersLost")
  }
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    start_date = date_range,
    end_date = end_date,
    ...
  )
}

#' Get top videos report
#' 
#' Retrieves performance metrics for individual videos, sorted by views.
#' 
#' @param date_range Date range string or start date
#' @param end_date End date (only needed if date_range is a specific start date)  
#' @param ids Channel identifier (default: "channel==MINE")
#' @param max_results Number of top videos to return (default: 10)
#' @param metrics Vector of metrics to include (default: views, likes, comments)
#' @param ... Additional arguments passed to get_report()
#' @return API response with top videos data
#' @export
#' @examples
#' \dontrun{
#' # Top 10 videos by views in last 30 days
#' get_top_videos("last_30_days")
#' 
#' # Top 25 videos with more metrics
#' get_top_videos("this_month", max_results = 25, 
#'                metrics = c("views", "likes", "comments", "shares", "estimatedMinutesWatched"))
#' }
get_top_videos <- function(date_range, end_date = NULL, ids = "channel==MINE",
                          max_results = 10, metrics = c("views", "likes", "comments"), ...) {
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    dimensions = "video",
    start_date = date_range,
    end_date = end_date,
    sort = "-views",  # Sort by views descending
    max_results = max_results,
    ...
  )
}

#' Get audience demographics report
#' 
#' Retrieves demographic breakdown of your audience by age and gender.
#' 
#' @param date_range Date range string or start date
#' @param end_date End date (only needed if date_range is a specific start date)
#' @param ids Channel identifier (default: "channel==MINE")  
#' @param dimension Demographic dimension: "ageGroup", "gender", or both (default: both)
#' @param metrics Vector of metrics to include (default: views, estimatedMinutesWatched)
#' @param ... Additional arguments passed to get_report()
#' @return API response with demographic data
#' @export
#' @examples
#' \dontrun{
#' # Full demographic breakdown
#' get_audience_demographics("last_90_days")
#' 
#' # Age groups only
#' get_audience_demographics("this_month", dimension = "ageGroup")
#' 
#' # Gender breakdown only  
#' get_audience_demographics("this_quarter", dimension = "gender")
#' }
get_audience_demographics <- function(date_range, end_date = NULL, ids = "channel==MINE",
                                    dimension = c("ageGroup", "gender"), 
                                    metrics = c("views", "estimatedMinutesWatched"), ...) {
  
  # Handle dimension parameter
  if (length(dimension) > 1) {
    dimensions <- paste(dimension, collapse = ",")
  } else {
    dimensions <- dimension
  }
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    dimensions = dimensions,
    start_date = date_range,
    end_date = end_date,
    ...
  )
}

#' Get geographic performance report
#' 
#' Retrieves performance metrics broken down by country or other geographic dimensions.
#' 
#' @param date_range Date range string or start date
#' @param end_date End date (only needed if date_range is a specific start date)
#' @param ids Channel identifier (default: "channel==MINE")
#' @param dimension Geographic dimension: "country", "province", or "city" (default: "country")
#' @param metrics Vector of metrics to include (default: views, estimatedMinutesWatched)
#' @param max_results Number of results to return (default: 25)
#' @param ... Additional arguments passed to get_report()
#' @return API response with geographic data
#' @export
#' @examples
#' \dontrun{
#' # Top countries by views
#' get_geographic_performance("last_30_days")
#' 
#' # US states/provinces (requires US-only data)
#' get_geographic_performance("this_month", dimension = "province", 
#'                           filters = "country==US")
#' }
get_geographic_performance <- function(date_range, end_date = NULL, ids = "channel==MINE",
                                     dimension = "country", 
                                     metrics = c("views", "estimatedMinutesWatched"),
                                     max_results = 25, ...) {
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    dimensions = dimension,
    start_date = date_range,
    end_date = end_date,
    sort = "-views",
    max_results = max_results,
    ...
  )
}


#' Get daily performance time series
#' 
#' Retrieves day-by-day performance metrics for trend analysis.
#' 
#' @param date_range Date range string or start date  
#' @param end_date End date (only needed if date_range is a specific start date)
#' @param ids Channel identifier (default: "channel==MINE")
#' @param metrics Vector of metrics to include (default: views, estimatedMinutesWatched)
#' @param ... Additional arguments passed to get_report()
#' @return API response with daily time series data
#' @export
#' @examples
#' \dontrun{
#' # Daily views for last 30 days
#' get_daily_performance("last_30_days")
#' 
#' # Daily performance with engagement metrics
#' get_daily_performance("this_month", metrics = c("views", "likes", "comments", "shares"))
#' }
get_daily_performance <- function(date_range, end_date = NULL, ids = "channel==MINE",
                                metrics = c("views", "estimatedMinutesWatched"), ...) {
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    dimensions = "day",
    start_date = date_range,
    end_date = end_date,
    sort = "day",  # Sort chronologically
    ...
  )
}

#' Get revenue report (requires monetary scope)
#' 
#' Retrieves revenue and monetization metrics.
#' 
#' @param date_range Date range string or start date
#' @param end_date End date (only needed if date_range is a specific start date)
#' @param ids Channel identifier (default: "channel==MINE")
#' @param currency Currency code (default: "USD")
#' @param include_cpm Logical. Include CPM metrics (default: TRUE)
#' @param ... Additional arguments passed to get_report()
#' @return API response with revenue data
#' @export
#' @examples
#' \dontrun{
#' # Revenue report for last month
#' get_revenue_report("last_month")
#' 
#' # Revenue in different currency
#' get_revenue_report("this_quarter", currency = "EUR")
#' }
get_revenue_report <- function(date_range, end_date = NULL, ids = "channel==MINE",
                             currency = "USD", include_cpm = TRUE, ...) {
  
  metrics <- c("estimatedRevenue", "adEarnings", "monetizedPlaybacks")
  
  if (include_cpm) {
    metrics <- c(metrics, "playbackBasedCpm", "impressionBasedCpm")
  }
  
  get_report(
    ids = ids,
    metrics = paste(metrics, collapse = ","),
    start_date = date_range,
    end_date = end_date,
    currency = currency,
    ...
  )
}

#' Update Groups
#'
#' @param resource_details Named nested list. Required. Must provide:
#' id, snippet title
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_PUT}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groups/update}
#'
#' @examples
#'
#' \dontrun{
#' update_group(list(id="ABZZzGSIAAA", snippet = list(title ="hello")))
#' }

update_group <- function (resource_details, ...) {
  assert_list(resource_details, .var.name = "resource_details")

  json_arg <- toJSON(resource_details, auto_unbox = TRUE)
  tubern_PUT("groups", body = json_arg, encode = "json", ...)
}

#' Add Group Item
#'
#' @param resource_details Named nested list. Required. Must provide:
#' groupId, resource.id
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_POST}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groupItems/insert}
#'
#' @examples
#'
#' \dontrun{
#' add_group_item(list(groupId = "", resource.id ="hello"))
#' }

add_group_item <- function (resource_details, ...) {
  assert_list(resource_details, .var.name = "resource_details")

  json_arg <- toJSON(resource_details, auto_unbox = TRUE)
  tubern_POST("groupItems", body = json_arg, encode = "json", ...)
}

context("Group Items endpoints")

test_that("add_group_item constructs correct API call", {
  resource_details <- list(groupId = "g1", resourceId = "r1")
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- add_group_item(resource_details)
  expect_equal(res$method, "POST")
  expect_equal(res$path, "groupItems")
  expect_null(res$query)
  expect_equal(res$body, jsonlite::toJSON(resource_details, auto_unbox = TRUE))
})

test_that("list_group_items constructs correct API call", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- list_group_items(group_id = "g1")
  expect_equal(res$method, "GET")
  expect_equal(res$path, "groupItems")
  expect_equal(res$query, c(groupId = "g1"))
  expect_null(res$body)
})

test_that("delete_group_item constructs correct API call", {
  stub <- function(method, path, query = NULL, body = NULL, ...) {
    list(method = method, path = path, query = query, body = body)
  }
  orig <- getFromNamespace(".api_request", ns = "tubern")
  on.exit(assignInNamespace(".api_request", orig, ns = "tubern"))
  assignInNamespace(".api_request", stub, ns = "tubern")
  res <- delete_group_item(id = "gi123")
  expect_equal(res$method, "DELETE")
  expect_equal(res$path, "groupItems")
  expect_equal(res$query, list(id = "gi123"))
  expect_equal(res$body, "")
})

#' Add Groups
#'
#' Creates a new channel group.
#'
#' @param resource_details Named nested list. Required. Must provide:
#' snippet with title (required), and optionally description and contentDetails with itemType
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_POST}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groups/insert}
#'
#' @examples
#'
#' \dontrun{
#' add_groups(list(
#'   snippet = list(
#'     title = "My Channel Group",
#'     description = "A group for organizing channels"
#'   ),
#'   contentDetails = list(itemType = "youtube#channel")
#' ))
#' }

add_groups <- function (resource_details, ...) {
  assert_list(resource_details, .var.name = "resource_details")

  json_arg <- toJSON(resource_details, auto_unbox = TRUE)
  tubern_POST("groups", body = json_arg, encode = "json", ...)
}

#' \pkg{tubern} provides access to the YouTube Analytics and Reporting API
#'
#' @importFrom httr GET POST PUT DELETE authenticate config stop_for_status upload_file content oauth_endpoints oauth_app oauth2.0_token
#' @importFrom jsonlite toJSON
#' @importFrom utils URLencode adist packageVersion write.csv
#' @importFrom stats median
#' @importFrom graphics barplot points
#' @importFrom rlang abort warn inform caller_env
#' @importFrom checkmate assert_character assert_string assert_flag assert_int assert_number assert_list check_string
#' @name tubern
"_PACKAGE"

# Internal: base URL for YouTube Analytics API
.api_base <- "https://youtubeanalytics.googleapis.com/v2"

# Internal: send API request to YouTube Analytics
# @param method character: one of "GET", "POST", "PUT", "DELETE"
# @param path character: API endpoint path (relative to base)
# @param query list: query parameters
# @param body optional body for POST/PUT/DELETE
# @param ... additional args passed to httr functions (e.g., encode)
# @return parsed content as list
.api_request <- function(method, path, query = NULL, body = NULL, ...) {
  .api_request_enhanced(method, path, query, body, ...)
}


#' Check if authentication token is in options
#'

yt_check_token <- function() {
  app_token <- getOption("google_token")
  if (is.null(app_token)) {
    tubern_abort(
      "No authentication token found. Run yt_oauth() to authenticate.",
      class = "auth"
    )
  }
}

#'
#' Base POST AND GET functions. Not exported.

#'
#' GET
#'
#' @param path path to specific API request URL
#' @param query query list
#' @param \dots Additional arguments passed to \code{\link[httr]{GET}}.
#' @return list

tubern_GET <- function(path, query = NULL, ...) {
  .api_request("GET", path, query = query, ...)
}

#'
#' POST
#'
#' @param path path to specific API request URL
#' @param query query list
#' @param body passing image through body
#' @param \dots Additional arguments passed to \code{\link[httr]{POST}}.
#'
#' @return list

tubern_POST <- function(path, query = NULL, body = "", ...) {
  .api_request("POST", path, query = query, body = body, ...)
}

#'
#' PUT
#'
#' @param path path to specific API request URL
#' @param query query list
#' @param body passing image through body
#' @param \dots Additional arguments passed to \code{\link[httr]{PUT}}.
#'
#' @return list

tubern_PUT <- function(path, query = NULL, body = "", ...) {
  .api_request("PUT", path, query = query, body = body, ...)
}

#'
#' DELETE
#'
#' @param path path to specific API request URL
#' @param query query list
#' @param body passing image through body
#' @param \dots Additional arguments passed to \code{\link[httr]{DELETE}}.
#'
#' @return list

tubern_DELETE <- function(path, query = NULL, body = "", ...) {
  .api_request("DELETE", path, query = query, body = body, ...)
}

context("API client wrappers")

test_that("tubern_GET errors when no token is set", {
  old_token <- getOption("google_token")
  options(google_token = NULL)
  expect_error(
    tubern:::tubern_GET("reports"),
    class = "tubern_auth_error"
  )
  options(google_token = old_token)
})

test_that("API base URL constant is correct", {
  expect_identical(
    tubern:::.api_base,
    "https://youtubeanalytics.googleapis.com/v2"
  )
})

test_that("yt_check_token throws tubern_auth_error when no token", {
  old_token <- getOption("google_token")
  options(google_token = NULL)
  expect_error(
    tubern:::yt_check_token(),
    class = "tubern_auth_error"
  )
  options(google_token = old_token)
})

#' Date Helper Functions for tubern
#'
#' Convenience functions for handling dates in YouTube Analytics API requests
#' @name date_helpers
NULL

#' Parse relative date strings to actual dates
#' 
#' @param date_string Character string like "last_30_days", "this_month", "2024-01-01", etc.
#' @return Date object
#' @keywords internal
#' @noRd
.parse_date_string <- function(date_string) {
  if (is.null(date_string)) return(NULL)
  
  # Handle absolute dates first
  if (grepl("^\\d{4}-\\d{2}-\\d{2}$", date_string)) {
    return(as.Date(date_string))
  }
  
  # Handle relative dates
  today <- Sys.Date()
  
  date_obj <- switch(tolower(date_string),
    # Days
    "yesterday" = today - 1,
    "today" = today,
    "last_7_days" = today - 7,
    "last_14_days" = today - 14,
    "last_30_days" = today - 30,
    "last_60_days" = today - 60,
    "last_90_days" = today - 90,
    
    # Weeks
    "this_week" = today - as.numeric(format(today, "%w")),
    "last_week" = today - as.numeric(format(today, "%w")) - 7,
    
    # Months
    "this_month" = as.Date(paste0(format(today, "%Y-%m"), "-01")),
    "last_month" = {
      first_this_month <- as.Date(paste0(format(today, "%Y-%m"), "-01"))
      seq(first_this_month, length = 2, by = "-1 month")[2]
    },
    "this_year" = as.Date(paste0(format(today, "%Y"), "-01-01")),
    "last_year" = as.Date(paste0(as.numeric(format(today, "%Y")) - 1, "-01-01")),
    
    # Quarters
    "this_quarter" = {
      quarter <- ceiling(as.numeric(format(today, "%m")) / 3)
      quarter_start_month <- (quarter - 1) * 3 + 1
      as.Date(paste0(format(today, "%Y"), "-", 
                     sprintf("%02d", quarter_start_month), "-01"))
    },
    "last_quarter" = {
      quarter <- ceiling(as.numeric(format(today, "%m")) / 3)
      if (quarter == 1) {
        # Last quarter of previous year
        as.Date(paste0(as.numeric(format(today, "%Y")) - 1, "-10-01"))
      } else {
        quarter_start_month <- (quarter - 2) * 3 + 1
        as.Date(paste0(format(today, "%Y"), "-", 
                       sprintf("%02d", quarter_start_month), "-01"))
      }
    },
    
    tubern_abort(
      paste0("Unrecognized date string: '", date_string, "'.",
             " Use YYYY-MM-DD format or relative dates like 'last_30_days', 'this_month', etc."),
      class = "parameter"
    )
  )
  
  return(date_obj)
}

#' Get end date for relative date ranges
#' 
#' @param start_date_string Character string for start date
#' @return Date object for appropriate end date
#' @keywords internal
#' @noRd
.get_end_date_for_range <- function(start_date_string) {
  if (is.null(start_date_string)) return(NULL)
  
  # Handle absolute dates - return same date
  if (grepl("^\\d{4}-\\d{2}-\\d{2}$", start_date_string)) {
    return(as.Date(start_date_string))
  }
  
  today <- Sys.Date()
  
  end_date <- switch(tolower(start_date_string),
    # Days
    "yesterday" = today - 1,
    "today" = today,
    "last_7_days" = today - 1,  # Last 7 days up to yesterday
    "last_14_days" = today - 1,
    "last_30_days" = today - 1,
    "last_60_days" = today - 1,
    "last_90_days" = today - 1,
    
    # Weeks
    "this_week" = today,
    "last_week" = today - as.numeric(format(today, "%w")) - 1,
    
    # Months
    "this_month" = today,
    "last_month" = {
      first_this_month <- as.Date(paste0(format(today, "%Y-%m"), "-01"))
      first_this_month - 1
    },
    "this_year" = today,
    "last_year" = as.Date(paste0(as.numeric(format(today, "%Y")) - 1, "-12-31")),
    
    # Quarters
    "this_quarter" = today,
    "last_quarter" = {
      quarter <- ceiling(as.numeric(format(today, "%m")) / 3)
      if (quarter == 1) {
        # End of last quarter of previous year
        as.Date(paste0(as.numeric(format(today, "%Y")) - 1, "-12-31"))
      } else {
        quarter_end_month <- (quarter - 2) * 3 + 3
        last_day <- switch(as.character(quarter_end_month),
          "3" = "31", "6" = "30", "9" = "30", "12" = "31"
        )
        as.Date(paste0(format(today, "%Y"), "-", 
                       sprintf("%02d", quarter_end_month), "-", last_day))
      }
    },
    
    # Default: return today
    today
  )
  
  return(end_date)
}

#' Resolve date range with support for relative dates
#' 
#' @param start_date Start date (can be relative like "last_30_days" or absolute like "2024-01-01") 
#' @param end_date End date (can be relative or absolute). If NULL and start_date is relative, will be calculated automatically
#' @return List with resolved start_date and end_date as YYYY-MM-DD strings
#' @export
#' @examples
#' # Relative date ranges
#' resolve_date_range("last_30_days")
#' resolve_date_range("this_month")
#' resolve_date_range("last_quarter")
#' 
#' # Absolute date ranges  
#' resolve_date_range("2024-01-01", "2024-01-31")
resolve_date_range <- function(start_date, end_date = NULL) {
  # Parse start date
  start_parsed <- .parse_date_string(start_date)
  
  # If end_date is not provided and start_date is relative, calculate end_date
  if (is.null(end_date) && !grepl("^\\d{4}-\\d{2}-\\d{2}$", start_date)) {
    end_parsed <- .get_end_date_for_range(start_date)
  } else if (!is.null(end_date)) {
    end_parsed <- .parse_date_string(end_date)
  } else {
    end_parsed <- start_parsed  # Same as start if absolute and no end specified
  }
  
  if (start_parsed > end_parsed) {
    tubern_abort(
      paste0("Resolved start_date (", format(start_parsed, "%Y-%m-%d"),
             ") is after end_date (", format(end_parsed, "%Y-%m-%d"), ")"),
      class = "parameter"
    )
  }
  
  if (end_parsed > Sys.Date()) {
    tubern_warn(
      "End date is in the future. YouTube Analytics data is typically delayed by 1-2 days.",
      class = "parameter"
    )
  }
  
  # Return as formatted strings
  list(
    start_date = format(start_parsed, "%Y-%m-%d"),
    end_date = format(end_parsed, "%Y-%m-%d")
  )
}

#' Get common date ranges
#' 
#' @return Named list of common date ranges
#' @export  
#' @examples
#' get_common_date_ranges()
get_common_date_ranges <- function() {
  list(
    "Last 7 days" = "last_7_days",
    "Last 30 days" = "last_30_days", 
    "Last 90 days" = "last_90_days",
    "This month" = "this_month",
    "Last month" = "last_month",
    "This quarter" = "this_quarter",
    "Last quarter" = "last_quarter", 
    "This year" = "this_year",
    "Last year" = "last_year",
    "Yesterday" = "yesterday",
    "This week" = "this_week",
    "Last week" = "last_week"
  )
}

#' Delete Group
#'
#' @param id  String. Required.
#' ``The id parameter specifies the YouTube group ID of the group that is being deleted.''
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_DELETE}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groups/delete}
#'
#' @examples
#'
#' \dontrun{
#' delete_group(id="ABZZzGSIAAA")
#' }

delete_group <- function (id, ...) {
  assert_string(id, .var.name = "id")

  querylist <- list(id = id)
  tubern_DELETE("groups", query = querylist, ...)
}

#' List Group Items
#'
#' @param group_id String. Required. ID of the group
#' @param \dots Additional arguments passed to \code{\link[tubern]{tubern_GET}}.
#'
#' @return named list
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/groupItems/list}
#'
#' @examples
#' \dontrun{
#' list_group_items(group_id = "vponEBg8hrR1yBUX0Hz66Uc5WMk/vyGp6PvFo4RvsFtPoIWeCReyIC8")
#' }

list_group_items <- function (group_id, ...) {
  assert_string(group_id, .var.name = "group_id")

  querylist <- c(groupId = group_id)
  tubern_GET("groupItems", querylist, ...)
}

#' Enhanced Error Handling for tubern
#'
#' Custom error classes and improved error handling for YouTube Analytics API
#' using rlang for modern error handling patterns.
#' @name error_handling
#' @importFrom rlang abort warn inform caller_env
NULL

#' Abort with tubern-specific error class
#'
#' @param message Error message
#' @param class Additional error class (e.g., "auth", "quota", "parameter", "api")
#' @param ... Additional data to include in the error condition
#' @param call The call to include in the error (default: caller's environment)
#' @return Never returns; always throws an error
#' @keywords internal
#' @noRd
tubern_abort <- function(message, class = NULL, ..., call = caller_env()) {
  full_class <- c(
    if (!is.null(class)) paste0("tubern_", class, "_error"),
    "tubern_error"
  )
  abort(message, class = full_class, ..., call = call)
}

#' Warn with tubern-specific warning class
#'
#' @param message Warning message
#' @param class Additional warning class
#' @param ... Additional data to include in the warning condition
#' @keywords internal
#' @noRd
tubern_warn <- function(message, class = NULL, ...) {
  full_class <- c(
    if (!is.null(class)) paste0("tubern_", class, "_warning"),
    "tubern_warning"
  )
  warn(message, class = full_class, ...)
}

#' Inform with tubern-specific message class
#'
#' @param message Informational message
#' @param class Additional message class
#' @param ... Additional data to include in the message condition
#' @keywords internal
#' @noRd
tubern_inform <- function(message, class = NULL, ...) {
  full_class <- c(
    if (!is.null(class)) paste0("tubern_", class, "_message"),
    "tubern_message"

  )
  inform(message, class = full_class, ...)
}

#' Execute with retry for transient errors
#'
#' Implements exponential backoff retry logic for transient API errors.
#'
#' @param expr Expression to evaluate
#' @param max_tries Maximum number of attempts (default: 3)
#' @param base_delay Base delay in seconds between retries (default: 1)
#' @param max_delay Maximum delay in seconds (default: 60)
#' @param retry_on HTTP status codes to retry on (default: 429, 500, 502, 503, 504)
#' @return Result of the expression if successful
#' @keywords internal
#' @noRd
with_retry <- function(expr,
                       max_tries = 3,
                       base_delay = 1,
                       max_delay = 60,
                       retry_on = c(429L, 500L, 502L, 503L, 504L)) {
  attempt <- 1
  last_error <- NULL


  while (attempt <= max_tries) {
    result <- tryCatch(
      {
        force(expr)
      },
      tubern_quota_error = function(e) {
        if (attempt < max_tries) {
          delay <- min(base_delay * (2^(attempt - 1)), max_delay)
          tubern_inform(
            sprintf("Rate limited. Retrying in %.1f seconds (attempt %d/%d)...",
                    delay, attempt, max_tries),
            class = "retry"
          )
          Sys.sleep(delay)
        }
        last_error <<- e
        NULL
      },
      tubern_api_error = function(e) {
        status <- e$status_code
        if (!is.null(status) && status %in% retry_on && attempt < max_tries) {
          delay <- min(base_delay * (2^(attempt - 1)), max_delay)
          tubern_inform(
            sprintf("Transient error (HTTP %d). Retrying in %.1f seconds (attempt %d/%d)...",
                    status, delay, attempt, max_tries),
            class = "retry"
          )
          Sys.sleep(delay)
          last_error <<- e
          return(NULL)
        }
        stop(e)
      },
      error = function(e) {
        stop(e)
      }
    )

    if (!is.null(result)) {
      return(result)
    }

    attempt <- attempt + 1
  }

  if (!is.null(last_error)) {
    stop(last_error)
  }

  tubern_abort("Maximum retry attempts exceeded", class = "api")
}

#' Enhanced API request function with better error handling
#'
#' @param method HTTP method
#' @param path API endpoint path
#' @param query Query parameters
#' @param body Request body
#' @param ... Additional parameters
#' @param retry Logical. Whether to use retry logic (default: TRUE)
#' @return API response
#' @keywords internal
#' @noRd
.api_request_enhanced <- function(method, path, query = NULL, body = NULL, ..., retry = TRUE) {
  yt_check_token()

  url <- paste0(.api_base, "/", path)
  fun <- switch(method,
                GET = GET,
                POST = POST,
                PUT = PUT,
                DELETE = DELETE,
                tubern_abort(paste("Unsupported HTTP method:", method), class = "parameter"))

  make_request <- function() {
    req <- tryCatch({
      fun(url,
          query = query,
          body = body,
          config(token = getOption("google_token")),
          ...)
    }, error = function(e) {
      tubern_abort(
        paste("Network error:", conditionMessage(e)),
        class = "network"
      )
    })

    .handle_api_response(req)
  }

  if (retry) {
    with_retry(make_request())
  } else {
    make_request()
  }
}

#' Handle API response and convert errors to rlang conditions
#'
#' @param req httr response object
#' @return Parsed content if successful
#' @keywords internal
#' @noRd
.handle_api_response <- function(req) {
  if (req$status_code == 200) {
    return(content(req))
  }

  error_content <- tryCatch(content(req), error = function(e) NULL)

  error_info <- .parse_api_error(req$status_code, error_content)

  tubern_abort(
    error_info$message,
    class = error_info$class,
    status_code = req$status_code,
    response = req,
    api_error = error_content
  )
}

#' Parse API error response into message and class
#'
#' @param status_code HTTP status code
#' @param error_content Parsed error content from response
#' @return List with message and class
#' @keywords internal
#' @noRd
.parse_api_error <- function(status_code, error_content) {
  error_details <- NULL
  if (!is.null(error_content) && !is.null(error_content$error$errors)) {
    error_details <- error_content$error$errors[[1]]
  }

  result <- switch(
    as.character(status_code),
    "400" = {
      msg <- if (!is.null(error_details$message)) {
        paste("Bad request:", error_details$message)
      } else {
        "Bad request. Check your parameters."
      }
      list(message = msg, class = "parameter")
    },
    "401" = list(
      message = paste(
        "Authentication failed.",
        "Run yt_oauth() to refresh your token or check your OAuth scopes."
      ),
      class = "auth"
    ),
    "403" = {
      reason <- error_details$reason
      if (!is.null(reason) && grepl("quotaExceeded|rateLimitExceeded", reason)) {
        list(
          message = paste(
            "API quota exceeded.",
            "Reduce request scope or wait before trying again.",
            "Check quota limits at https://console.cloud.google.com"
          ),
          class = "quota"
        )
      } else {
        list(
          message = "Access forbidden. Check your permissions and API settings.",
          class = "auth"
        )
      }
    },
    "404" = list(
      message = paste(
        "Resource not found. Possible causes:",
        "- YouTube Analytics API not enabled in Google Cloud project",
        "- Channel/content owner ID doesn't exist or is inaccessible",
        "- Incorrect authentication scopes",
        sep = "\n"
      ),
      class = "api"
    ),
    "429" = list(
      message = "Too many requests. Please slow down and try again.",
      class = "quota"
    ),
    "500" = list(
      message = "YouTube API server error. Please try again later.",
      class = "api"
    ),
    "502" = list(
      message = "YouTube API bad gateway error. Please try again.",
      class = "api"
    ),
    "503" = list(
      message = "YouTube API service unavailable. Please try again later.",
      class = "api"
    ),
    "504" = list(
      message = "YouTube API gateway timeout. Please try again.",
      class = "api"
    ),
    list(
      message = paste("HTTP", status_code, "error occurred"),
      class = "api"
    )
  )

  result
}

#' Check API quota status and provide guidance
#'
#' @export
#' @return Message about current quota usage (if available)
#' @examples
#' \dontrun{
#' check_api_quota()
#' }
check_api_quota <- function() {
  tryCatch({
    result <- get_report(
      ids = "channel==MINE",
      metrics = "views",
      start_date = format(Sys.Date() - 1, "%Y-%m-%d"),
      end_date = format(Sys.Date() - 1, "%Y-%m-%d"),
      max_results = 1
    )
    tubern_inform("API access is working normally.")
  }, tubern_quota_error = function(e) {
    tubern_inform(c(
      "Quota exceeded. Consider:",
      "- Reducing the date range of your requests",
      "- Using fewer metrics or dimensions",
      "- Implementing caching for repeated requests",
      "- Checking your Google Cloud Console quota limits"
    ))
  }, tubern_auth_error = function(e) {
    tubern_inform(c(
      "Authentication issue. Try:",
      "- Running yt_oauth() again",
      "- Checking your OAuth scopes",
      "- Verifying your Google Cloud project settings"
    ))
  }, error = function(e) {
    tubern_inform(paste("API check failed:", conditionMessage(e)))
  })
}

#' Diagnose common tubern issues
#'
#' @export
#' @return Diagnostic information about tubern setup
#' @examples
#' \dontrun{
#' diagnose_tubern()
#' }
diagnose_tubern <- function() {
  cat("=== tubern Diagnostic Report ===\n\n")

  cat("1. Authentication Status:\n")
  token <- getOption("google_token")
  if (is.null(token)) {
    cat("   X No authentication token found\n")
    cat("   -> Run yt_oauth() to authenticate\n\n")
  } else {
    cat("   OK Authentication token present\n")

    tryCatch({
      yt_check_token()
      cat("   OK Token validation passed\n")
    }, error = function(e) {
      cat("   X Token validation failed\n")
      cat("   -> Run yt_oauth() to refresh your token\n")
    })
    cat("\n")
  }

  cat("2. Network Connectivity:\n")
  tryCatch({
    test_req <- GET("https://www.googleapis.com")
    if (test_req$status_code == 200) {
      cat("   OK Network connection to Google APIs working\n")
    } else {
      cat("   X Network connection issues detected\n")
    }
  }, error = function(e) {
    cat("   X Network connection failed:", conditionMessage(e), "\n")
    cat("   -> Check your internet connection and proxy settings\n")
  })
  cat("\n")

  cat("3. YouTube Analytics API Access:\n")
  if (!is.null(token)) {
    tryCatch({
      check_api_quota()
    }, error = function(e) {
      cat("   X API access test failed\n")
      cat("   -> Error:", conditionMessage(e), "\n")
    })
  } else {
    cat("   - Skipped (no authentication token)\n")
  }
  cat("\n")

  cat("4. Package Information:\n")
  cat("   tubern version:", as.character(packageVersion("tubern")), "\n")
  cat("   R version:", R.version.string, "\n")

  cat("\n=== End Diagnostic Report ===\n")
}

context("Error handling")

test_that("tubern_abort creates proper error class", {
  err <- tryCatch(
    tubern:::tubern_abort("test error", class = "auth"),
    error = function(e) e
  )
  expect_s3_class(err, "tubern_auth_error")
  expect_s3_class(err, "tubern_error")
  expect_s3_class(err, "rlang_error")
})

test_that("tubern_abort includes custom class in message", {
  err <- tryCatch(
    tubern:::tubern_abort("custom message", class = "quota"),
    error = function(e) e
  )
  expect_s3_class(err, "tubern_quota_error")
})

test_that("tubern_warn creates warning with proper class", {
  expect_warning(
    tubern:::tubern_warn("test warning", class = "parameter"),
    class = "tubern_parameter_warning"
  )
})

test_that(".parse_api_error handles 400 errors", {
  result <- tubern:::.parse_api_error(400, NULL)
  expect_equal(result$class, "parameter")
  expect_true(grepl("Bad request", result$message))
})

test_that(".parse_api_error handles 401 errors", {
  result <- tubern:::.parse_api_error(401, NULL)
  expect_equal(result$class, "auth")
  expect_true(grepl("Authentication", result$message))
})

test_that(".parse_api_error handles 403 quota errors", {
  error_content <- list(
    error = list(
      errors = list(
        list(reason = "quotaExceeded", message = "Quota exceeded")
      )
    )
  )
  result <- tubern:::.parse_api_error(403, error_content)
  expect_equal(result$class, "quota")
})

test_that(".parse_api_error handles 403 permission errors", {
  result <- tubern:::.parse_api_error(403, NULL)
  expect_equal(result$class, "auth")
  expect_true(grepl("forbidden", result$message, ignore.case = TRUE))
})

test_that(".parse_api_error handles 404 errors", {
  result <- tubern:::.parse_api_error(404, NULL)
  expect_equal(result$class, "api")
  expect_true(grepl("not found", result$message, ignore.case = TRUE))
})

test_that(".parse_api_error handles 429 errors", {
  result <- tubern:::.parse_api_error(429, NULL)
  expect_equal(result$class, "quota")
})

test_that(".parse_api_error handles 500 errors", {
  result <- tubern:::.parse_api_error(500, NULL)
  expect_equal(result$class, "api")
  expect_true(grepl("server error", result$message, ignore.case = TRUE))
})

test_that(".parse_api_error handles unknown errors", {
  result <- tubern:::.parse_api_error(418, NULL)
  expect_equal(result$class, "api")
  expect_true(grepl("418", result$message))
})

test_that("error classes can be caught specifically", {
  caught <- FALSE
  tryCatch(
    tubern:::tubern_abort("auth error", class = "auth"),
    tubern_auth_error = function(e) {
      caught <<- TRUE
    }
  )
  expect_true(caught)
})

test_that("error classes can be caught generically", {
  caught <- FALSE
  tryCatch(
    tubern:::tubern_abort("some error", class = "parameter"),
    tubern_error = function(e) {
      caught <<- TRUE
    }
  )
  expect_true(caught)
})

#' Data Transformation Utilities
#'
#' Functions to transform YouTube Analytics API responses into common R data formats
#' @name data_transformation
NULL

# Declare .data as global variable to avoid R CMD check NOTE
utils::globalVariables(".data")

#' Convert API response to data frame
#' 
#' Transforms YouTube Analytics API response into a clean data.frame with proper column names and types.
#' 
#' @param api_response List returned from get_report() or other API functions
#' @param clean_names Logical. Clean column names by removing special characters (default: TRUE)
#' @param parse_dates Logical. Parse date columns to Date objects (default: TRUE)
#' @return data.frame with transformed data, or NULL if no data available
#' @export
#' @examples
#' \dontrun{
#' # Get data and convert to data.frame
#' report <- get_channel_overview("last_30_days")
#' df <- yt_to_dataframe(report)
#' 
#' # Keep original column names
#' df <- yt_to_dataframe(report, clean_names = FALSE)
#' }
yt_to_dataframe <- function(api_response, clean_names = TRUE, parse_dates = TRUE) {
  
  if (is.null(api_response) || !is.list(api_response)) {
    tubern_warn("Invalid API response provided")
    return(NULL)
  }
  
  if (is.null(api_response$rows) || length(api_response$rows) == 0) {
    tubern_inform("No data available in the API response")
    return(data.frame())
  }
  
  if (is.null(api_response$columnHeaders)) {
    tubern_warn("No column headers found in API response")
    return(NULL)
  }
  
  headers <- vapply(api_response$columnHeaders, function(x) x$name, character(1))
  
  # Convert rows to matrix then data.frame
  data_matrix <- do.call(rbind, api_response$rows)
  df <- as.data.frame(data_matrix)
  names(df) <- headers
  
  # Clean column names if requested
  if (clean_names) {
    names(df) <- .clean_column_names(names(df))
  }
  
  # Parse data types
  df <- .parse_column_types(df, api_response$columnHeaders, parse_dates)
  
  # Add metadata as attributes
  attr(df, "query") <- list(
    start_date = api_response$query$startDate,
    end_date = api_response$query$endDate,
    metrics = api_response$query$metrics,
    dimensions = api_response$query$dimensions
  )
  
  return(df)
}

#' Clean column names for R data.frame
#' @param names Character vector of column names
#' @return Character vector of cleaned names
#' @keywords internal
#' @noRd
.clean_column_names <- function(names) {
  # Convert camelCase to snake_case
  names <- gsub("([a-z])([A-Z])", "\\1_\\2", names)
  names <- tolower(names)
  
  # Replace remaining special characters with underscores
  names <- gsub("[^a-z0-9_]", "_", names)
  
  # Remove duplicate underscores
  names <- gsub("_+", "_", names)
  
  # Remove leading/trailing underscores
  names <- gsub("^_|_$", "", names)
  
  return(names)
}

#' Parse column types based on API metadata
#' @param df Data.frame to parse
#' @param column_headers Column headers from API response
#' @param parse_dates Whether to parse date columns
#' @return Data.frame with parsed column types
#' @keywords internal
#' @noRd
.parse_column_types <- function(df, column_headers, parse_dates) {
  
  for (i in seq_along(column_headers)) {
    col_name <- names(df)[i]
    col_type <- column_headers[[i]]$dataType
    
    if (col_type == "INTEGER") {
      df[[col_name]] <- as.numeric(df[[col_name]])
    } else if (col_type == "FLOAT") {
      df[[col_name]] <- as.numeric(df[[col_name]])
    } else if (col_type == "STRING") {
      df[[col_name]] <- as.character(df[[col_name]])
    }
    
    # Parse date columns if requested
    if (parse_dates && (col_name %in% c("day", "month") || grepl("date", col_name, ignore.case = TRUE))) {
      date_parsed <- tryCatch({
        as.Date(df[[col_name]])
      }, error = function(e) NULL)
      
      if (!is.null(date_parsed)) {
        df[[col_name]] <- date_parsed
      }
    }
  }
  
  return(df)
}

#' Convert API response to tibble (if tibble is available)
#' 
#' @param api_response API response from YouTube Analytics
#' @param ... Additional arguments passed to yt_to_dataframe()
#' @return tibble or data.frame if tibble not available
#' @export
#' @examples
#' \dontrun{
#' report <- get_top_videos("last_7_days")
#' tbl <- yt_to_tibble(report)
#' }
yt_to_tibble <- function(api_response, ...) {
  df <- yt_to_dataframe(api_response, ...)
  
  if (is.null(df)) return(NULL)
  
  if (requireNamespace("tibble", quietly = TRUE)) {
    return(tibble::as_tibble(df))
  } else {
    tubern_inform("Install 'tibble' package for tibble output format")
    return(df)
  }
}

#' Extract summary statistics from API response
#' 
#' @param api_response API response from YouTube Analytics
#' @return Named list with summary statistics
#' @export
#' @examples
#' \dontrun{
#' report <- get_channel_overview("last_30_days")
#' summary <- yt_extract_summary(report)
#' print(summary)
#' }
yt_extract_summary <- function(api_response) {
  if (is.null(api_response) || is.null(api_response$rows)) {
    return(list(total_rows = 0))
  }
  
  df <- yt_to_dataframe(api_response, clean_names = FALSE, parse_dates = FALSE)
  if (is.null(df) || nrow(df) == 0) {
    return(list(total_rows = 0))
  }
  
  summary_list <- list(
    total_rows = nrow(df),
    columns = ncol(df),
    column_names = names(df)
  )
  
  # Add numeric column summaries
  numeric_cols <- vapply(df, is.numeric, logical(1))
  if (any(numeric_cols)) {
    summary_list$numeric_summary <- lapply(df[numeric_cols], function(x) {
      list(
        total = sum(x, na.rm = TRUE),
        mean = mean(x, na.rm = TRUE),
        median = median(x, na.rm = TRUE),
        min = min(x, na.rm = TRUE),
        max = max(x, na.rm = TRUE)
      )
    })
  }
  
  return(summary_list)
}

#' Export data to CSV
#' 
#' @param api_response API response from YouTube Analytics
#' @param filename Output filename (default: auto-generated based on timestamp)
#' @param ... Additional arguments passed to yt_to_dataframe()
#' @return Path to saved file
#' @export
#' @examples
#' \dontrun{
#' report <- get_daily_performance("last_30_days")
#' file_path <- yt_export_csv(report, "daily_performance.csv")
#' }
yt_export_csv <- function(api_response, filename = NULL, ...) {
  df <- yt_to_dataframe(api_response, ...)
  
  if (is.null(df) || nrow(df) == 0) {
    tubern_abort("No data to export", class = "parameter")
  }
  
  if (is.null(filename)) {
    timestamp <- format(Sys.time(), "%Y%m%d_%H%M%S")
    filename <- paste0("youtube_analytics_", timestamp, ".csv")
  }
  
  # Ensure .csv extension
  if (!grepl("\\.csv$", filename, ignore.case = TRUE)) {
    filename <- paste0(filename, ".csv")
  }
  
  write.csv(df, filename, row.names = FALSE)
  tubern_inform(paste("Data exported to:", filename))
  return(filename)
}

#' Create a quick visualization of the data (if ggplot2 is available)
#' 
#' @param api_response API response from YouTube Analytics
#' @param x_col Column name for x-axis (auto-detected if NULL)
#' @param y_col Column name for y-axis (auto-detected if NULL)
#' @param chart_type Type of chart: "line", "bar", "point" (default: auto)
#' @return ggplot object or base R plot if ggplot2 not available
#' @export
#' @examples
#' \dontrun{
#' # Daily views over time
#' daily_report <- get_daily_performance("last_30_days")
#' yt_quick_plot(daily_report)
#' 
#' # Top videos by views
#' top_videos <- get_top_videos("last_7_days")
#' yt_quick_plot(top_videos, chart_type = "bar")
#' }
yt_quick_plot <- function(api_response, x_col = NULL, y_col = NULL, chart_type = "auto") {
  df <- yt_to_dataframe(api_response)
  
  if (is.null(df) || nrow(df) == 0) {
    tubern_abort("No data to plot", class = "parameter")
  }
  
  # Auto-detect columns if not specified
  if (is.null(x_col)) {
    # Look for date columns first, then dimension columns
    date_cols <- names(df)[vapply(df, function(x) inherits(x, "Date"), logical(1))]
    if (length(date_cols) > 0) {
      x_col <- date_cols[1]
    } else {
      # Use first non-numeric column
      x_col <- names(df)[!vapply(df, is.numeric, logical(1))][1]
    }
  }
  
  if (is.null(y_col)) {
    # Use first numeric column
    numeric_cols <- names(df)[vapply(df, is.numeric, logical(1))]
    if (length(numeric_cols) > 0) {
      y_col <- numeric_cols[1]
    } else {
      tubern_abort("No numeric columns found for plotting", class = "parameter")
    }
  }
  
  # Auto-detect chart type
  if (chart_type == "auto") {
    if (inherits(df[[x_col]], "Date")) {
      chart_type <- "line"
    } else {
      chart_type <- "bar"
    }
  }
  
  if (requireNamespace("ggplot2", quietly = TRUE)) {
    .create_ggplot(df, x_col, y_col, chart_type)
  } else {
    tubern_inform("Install 'ggplot2' package for better plots. Using base R plot.")
    .create_base_plot(df, x_col, y_col, chart_type)
  }
}

#' Create ggplot visualization
#' @keywords internal
#' @noRd
.create_ggplot <- function(df, x_col, y_col, chart_type) {
  p <- ggplot2::ggplot(df, ggplot2::aes(x = .data[[x_col]], y = .data[[y_col]]))
  
  if (chart_type == "line") {
    p <- p + ggplot2::geom_line() + ggplot2::geom_point()
  } else if (chart_type == "bar") {
    p <- p + ggplot2::geom_col()
  } else if (chart_type == "point") {
    p <- p + ggplot2::geom_point()
  }
  
  p <- p + 
    ggplot2::labs(
      title = paste("YouTube Analytics:", y_col, "by", x_col),
      x = x_col,
      y = y_col
    ) +
    ggplot2::theme_minimal()
  
  if (!inherits(df[[x_col]], "Date") && chart_type == "bar") {
    p <- p + ggplot2::coord_flip()
  }
  
  return(p)
}

#' Create base R plot
#' @keywords internal
#' @noRd  
.create_base_plot <- function(df, x_col, y_col, chart_type) {
  if (chart_type == "line" && inherits(df[[x_col]], "Date")) {
    plot(df[[x_col]], df[[y_col]], type = "l", 
         xlab = x_col, ylab = y_col,
         main = paste("YouTube Analytics:", y_col, "by", x_col))
    points(df[[x_col]], df[[y_col]], pch = 16)
  } else if (chart_type == "bar") {
    barplot(df[[y_col]], names.arg = df[[x_col]],
            xlab = x_col, ylab = y_col,
            main = paste("YouTube Analytics:", y_col, "by", x_col),
            las = 2)
  } else {
    plot(df[[x_col]], df[[y_col]],
         xlab = x_col, ylab = y_col,
         main = paste("YouTube Analytics:", y_col, "by", x_col))
  }
}

context("Data transformation functions")

test_that("yt_to_dataframe handles parse_dates=FALSE correctly", {
  mock_response <- mock_api_response(
    rows = list(
      c("2023-01-01", "100"),
      c("2023-01-02", "200")
    ),
    column_headers = list(
      list(name = "day", dataType = "STRING"),
      list(name = "views", dataType = "INTEGER")
    ),
    query = list(
      startDate = "2023-01-01",
      endDate = "2023-01-02",
      metrics = "views",
      dimensions = "day"
    )
  )

  df_with_dates <- yt_to_dataframe(mock_response, parse_dates = TRUE)
  expect_s3_class(df_with_dates$day, "Date")

  df_without_dates <- yt_to_dataframe(mock_response, parse_dates = FALSE)
  expect_type(df_without_dates$day, "character")
})

test_that("yt_to_dataframe handles clean_names correctly", {
  mock_response <- mock_api_response(
    rows = list(c("123.45")),
    column_headers = list(
      list(name = "averageViewDuration", dataType = "FLOAT")
    ),
    query = list()
  )

  df_clean <- yt_to_dataframe(mock_response, clean_names = TRUE)
  expect_true("average_view_duration" %in% names(df_clean))

  df_original <- yt_to_dataframe(mock_response, clean_names = FALSE)
  expect_true("averageViewDuration" %in% names(df_original))
})

test_that("yt_to_dataframe returns NULL for invalid input", {
  expect_warning(result <- yt_to_dataframe(NULL))
  expect_null(result)

  expect_warning(result <- yt_to_dataframe("not a list"))
  expect_null(result)
})

test_that("yt_to_dataframe returns empty data.frame for empty rows", {
  mock_response <- list(
    columnHeaders = list(list(name = "views", dataType = "INTEGER")),
    rows = list()
  )

  expect_message(result <- yt_to_dataframe(mock_response))
  expect_true(is.data.frame(result))
  expect_equal(nrow(result), 0)
})

test_that("yt_extract_summary returns correct structure", {
  mock_response <- mock_api_response()

  summary <- yt_extract_summary(mock_response)
  expect_type(summary, "list")
  expect_equal(summary$total_rows, 2)
  expect_equal(summary$columns, 2)
})

test_that("yt_extract_summary handles empty response", {
  mock_response <- list(rows = NULL)
  summary <- yt_extract_summary(mock_response)
  expect_equal(summary$total_rows, 0)
})

test_that("yt_export_csv errors on empty data", {
  mock_response <- list(
    columnHeaders = list(list(name = "views", dataType = "INTEGER")),
    rows = list()
  )
  expect_error(
    yt_export_csv(mock_response),
    class = "tubern_parameter_error"
  )
})

test_that("yt_quick_plot errors on empty data", {
  mock_response <- list(
    columnHeaders = list(list(name = "views", dataType = "INTEGER")),
    rows = list()
  )
  expect_error(
    yt_quick_plot(mock_response),
    class = "tubern_parameter_error"
  )
})

test_that(".clean_column_names converts camelCase to snake_case", {
  result <- tubern:::.clean_column_names("averageViewDuration")
  expect_equal(result, "average_view_duration")

  result <- tubern:::.clean_column_names("estimatedMinutesWatched")
  expect_equal(result, "estimated_minutes_watched")
})

test_that(".parse_column_types handles INTEGER type", {
  df <- data.frame(views = "100", stringsAsFactors = FALSE)
  headers <- list(list(name = "views", dataType = "INTEGER"))

  result <- tubern:::.parse_column_types(df, headers, parse_dates = FALSE)
  expect_type(result$views, "double")
  expect_equal(result$views, 100)
})

test_that(".parse_column_types handles FLOAT type", {
  df <- data.frame(rate = "0.5", stringsAsFactors = FALSE)
  headers <- list(list(name = "rate", dataType = "FLOAT"))

  result <- tubern:::.parse_column_types(df, headers, parse_dates = FALSE)
  expect_type(result$rate, "double")
  expect_equal(result$rate, 0.5)
})

#' Set up Authorization
#'
#' Simplified OAuth2 setup for YouTube Analytics API. This function will automatically
#' detect the required scope based on your needs and provide helpful setup guidance.
#'
#' The function looks for .httr-oauth in the working directory. If it doesn't find it,
#' it expects an application ID and a secret. The function launches a browser to allow
#' you to authorize the application.
#'
#' @param app_id Client ID from Google Cloud Console; required; no default
#' @param app_secret Client secret from Google Cloud Console; required; no default
#' @param scope Character. One of:
#'   \itemize{
#'     \item \code{"analytics"} - Basic analytics data (views, likes, etc.) - Default
#'     \item \code{"monetary"} - Includes revenue and monetization data
#'     \item \code{"auto"} - Automatically detect scope based on first API call
#'   }
#' @param token Path to file containing the token. Default is \code{.httr-oauth} in working directory.
#' @param setup_guide Logical. Show setup guide for first-time users (default: TRUE for interactive sessions)
#' @param \dots Additional arguments passed to \code{\link[httr]{oauth2.0_token}}
#'
#' @return Sets the google_token option and saves .httr-oauth in working directory
#'
#' @export
#'
#' @references \url{https://developers.google.com/youtube/analytics/reference/}
#'
#' @examples
#'  \dontrun{
#'    # Basic setup (will show setup guide first time)
#'    yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret")
#'
#'    # Setup with monetary scope for revenue data
#'    yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret",
#'             scope = "monetary")
#'
#'    # Skip setup guide
#'    yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret",
#'             setup_guide = FALSE)
#' }

yt_oauth <- function (app_id = NULL, app_secret = NULL,
                      scope = "analytics", token = ".httr-oauth",
                      setup_guide = interactive(), ...) {

  assert_string(scope, .var.name = "scope")
  assert_string(token, .var.name = "token")
  assert_flag(setup_guide, .var.name = "setup_guide")

  if (setup_guide && !file.exists(token) && is.null(app_id)) {
    .show_oauth_setup_guide()
    return(invisible())
  }

  if (file.exists(token)) {
    google_token <- tryCatch(
      suppressWarnings(readRDS(token)),
      error = function(e) {
        tubern_abort(
          c(
            paste0("Unable to read token from: ", token),
            "The token file may be corrupted. Delete it and run yt_oauth() again."
          ),
          class = "auth"
        )
      }
    )

    google_token <- google_token[[1]]

    tryCatch({
      if (!is.null(google_token) && !is.null(google_token$credentials)) {
        tubern_inform("Using existing authentication token")
        options(google_token = google_token)
        return(invisible())
      }
    }, error = function(e) {
      tubern_warn("Existing token appears invalid, creating new one...")
    })
  }

  if (is.null(app_id) || is.null(app_secret)) {
    tubern_abort(
      c(
        "OAuth credentials required. Please provide app_id and app_secret.",
        "Need help? Run yt_oauth() with no parameters to see the setup guide."
      ),
      class = "auth"
    )
  }

  assert_string(app_id, .var.name = "app_id")
  assert_string(app_secret, .var.name = "app_secret")

  if (!grepl("\\.apps\\.googleusercontent\\.com$", app_id)) {
    tubern_warn("app_id should end with '.apps.googleusercontent.com'")
  }

  myapp <- oauth_app("google", key = app_id, secret = app_secret)

  scopes <- .get_oauth_scopes(scope)

  tubern_inform(paste("Setting up OAuth with scope:", scope))
  tubern_inform("Opening browser for authentication...")

  google_token <- tryCatch({
    oauth2.0_token(
      oauth_endpoints("google"),
      myapp,
      scope = scopes,
      ...
    )
  }, error = function(e) {
    tubern_abort(
      c(
        paste("OAuth setup failed:", conditionMessage(e)),
        "Check your app_id and app_secret, and ensure YouTube Analytics API is enabled."
      ),
      class = "auth"
    )
  })

  options(google_token = google_token)
  tubern_inform("Authentication successful!")

  tryCatch({
    yt_check_token()
    tubern_inform("Token validation successful")
  }, error = function(e) {
    tubern_warn(
      "Token created but validation failed. You may need to enable YouTube Analytics API in Google Cloud Console."
    )
  })
}

#' Get OAuth scopes based on scope parameter
#' @param scope Scope parameter from yt_oauth
#' @return Character vector of OAuth scope URLs
#' @keywords internal
#' @noRd
.get_oauth_scopes <- function(scope) {
  switch(scope,
    "analytics" = c(
      "https://www.googleapis.com/auth/yt-analytics.readonly",
      "https://www.googleapis.com/auth/youtube.readonly"
    ),
    "monetary" = c(
      "https://www.googleapis.com/auth/yt-analytics-monetary.readonly",
      "https://www.googleapis.com/auth/youtube.readonly"
    ),
    "auto" = c(
      "https://www.googleapis.com/auth/yt-analytics.readonly",
      "https://www.googleapis.com/auth/youtube.readonly"
    ),
    tubern_abort(
      "Invalid scope. Use 'analytics', 'monetary', or 'auto'",
      class = "parameter"
    )
  )
}

#' Show OAuth setup guide for first-time users
#' @keywords internal
#' @noRd
.show_oauth_setup_guide <- function() {
  cat("=== YouTube Analytics API Setup Guide ===\n\n")

  cat("To use tubern, you need to set up OAuth2 credentials:\n\n")

  cat("1. Go to Google Cloud Console: https://console.cloud.google.com\n")
  cat("2. Create a new project or select an existing one\n")
  cat("3. Enable the YouTube Analytics API:\n")
  cat("   - Go to 'APIs & Services' > 'Library'\n")
  cat("   - Search for 'YouTube Analytics API'\n")
  cat("   - Click 'Enable'\n\n")

  cat("4. Create OAuth2 credentials:\n")
  cat("   - Go to 'APIs & Services' > 'Credentials'\n")
  cat("   - Click '+ CREATE CREDENTIALS' > 'OAuth client ID'\n")
  cat("   - Choose 'Desktop application'\n")
  cat("   - Give it a name (e.g., 'tubern R client')\n")
  cat("   - Click 'Create'\n\n")

  cat("5. Copy your credentials and run:\n")
  cat("   yt_oauth('your-client-id.apps.googleusercontent.com', 'your-client-secret')\n\n")

  cat("Need help? Visit: https://developers.google.com/youtube/analytics/registering_an_application\n")
  cat("===========================================\n")
}

# https://github.com/jimhester/lintr
if (requireNamespace("lintr", quietly = TRUE)) {
  context("lints")
  test_that("Package Style", {
    skip_if_not_installed("lintr")
    skip_on_cran()
    
    # Try to find package root directory
    pkg_root <- tryCatch({
      # Look for DESCRIPTION file going up the directory tree
      current_dir <- getwd()
      while (!file.exists(file.path(current_dir, "DESCRIPTION")) && current_dir != dirname(current_dir)) {
        current_dir <- dirname(current_dir)
      }
      if (file.exists(file.path(current_dir, "DESCRIPTION"))) {
        current_dir
      } else {
        NULL
      }
    }, error = function(e) NULL)
    
    if (is.null(pkg_root)) {
      skip("Could not locate package root for linting")
    }
    
    # Run lintr on the package root
    tryCatch({
      lintr::expect_lint_free(path = pkg_root, cache = TRUE)
    }, warning = function(w) {
      # Convert warnings to skips to avoid CI failures
      skip(paste("Linting warning:", w$message))
    }, error = function(e) {
      skip(paste("Could not run linter:", e$message))
    })
  })
}

---
title: "tubern: Comprehensive YouTube Analytics Examples"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{tubern: Comprehensive YouTube Analytics Examples}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)
```

# tubern: Enhanced YouTube Analytics for R

This vignette demonstrates the enhanced features of tubern, showing you how to efficiently retrieve, analyze, and visualize YouTube Analytics data.

## Quick Setup

```{r setup}
library(tubern)

# First-time setup - shows setup guide
yt_oauth()

# Or with your credentials
yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret")
```

## New Features Overview

### 1. Relative Date Ranges

No more manual date calculations! Use intuitive relative dates:

```{r dates}
# Get common date range options
get_common_date_ranges()

# Use relative dates in reports
report <- get_report(
  ids = "channel==MINE",
  metrics = "views,likes",
  start_date = "last_30_days"  # End date calculated automatically
)

# More examples
get_report(ids = "channel==MINE", metrics = "views", start_date = "this_month")
get_report(ids = "channel==MINE", metrics = "views", start_date = "last_quarter")
get_report(ids = "channel==MINE", metrics = "views", start_date = "yesterday")
```

### 2. Helper Functions for Common Reports

#### Channel Overview
```{r channel_overview}
# Get comprehensive channel performance
overview <- get_channel_overview("last_30_days")

# Customize what metrics to include
overview <- get_channel_overview(
  "this_month", 
  include_engagement = TRUE,
  include_subscribers = TRUE
)
```

#### Top Performing Videos
```{r top_videos}
# Get top 10 videos by views
top_videos <- get_top_videos("last_7_days")

# Get more videos with custom metrics
top_videos <- get_top_videos(
  "last_30_days",
  max_results = 25,
  metrics = c("views", "likes", "comments", "shares", "estimatedMinutesWatched")
)
```

#### Audience Demographics
```{r demographics}
# Full demographic breakdown
demographics <- get_audience_demographics("last_90_days")

# Age groups only
age_data <- get_audience_demographics("this_month", dimension = "ageGroup")

# Gender breakdown only
gender_data <- get_audience_demographics("this_quarter", dimension = "gender")
```

#### Geographic Performance
```{r geographic}
# Top countries by views
geo_performance <- get_geographic_performance("last_30_days")

# US states/provinces (requires filtering to US)
us_states <- get_geographic_performance(
  "this_month", 
  dimension = "province",
  filters = "country==US"
)
```

#### Device and Platform Analysis
```{r devices}
# Performance by device type - use get_report directly
device_performance <- get_report(
  ids = "channel==MINE",
  metrics = "views,estimatedMinutesWatched",
  dimensions = "deviceType",
  start_date = "last_30_days"
)

# Operating system breakdown
os_performance <- get_report(
  ids = "channel==MINE",
  metrics = "views,estimatedMinutesWatched", 
  dimensions = "operatingSystem",
  start_date = "this_month"
)

# Both device and OS
device_os <- get_report(
  ids = "channel==MINE",
  metrics = "views,estimatedMinutesWatched",
  dimensions = "deviceType,operatingSystem",
  start_date = "last_7_days"
)
```

#### Daily Time Series Data
```{r daily}
# Daily performance trends
daily_data <- get_daily_performance("last_30_days")

# With engagement metrics
daily_engagement <- get_daily_performance(
  "this_month",
  metrics = c("views", "likes", "comments", "shares")
)
```

#### Revenue Reports (Requires Monetary Scope)
```{r revenue}
# Set up OAuth with monetary scope first
yt_oauth("your-client-id", "your-client-secret", scope = "monetary")

# Get revenue data
revenue <- get_revenue_report("last_month")

# Revenue in different currency
revenue_eur <- get_revenue_report("this_quarter", currency = "EUR")
```

### 3. Enhanced Parameter Validation

tubern now provides helpful validation and suggestions:

```{r validation}
# Invalid metrics get helpful suggestions
# get_report(ids = "channel==MINE", metrics = "vews", start_date = "last_7_days")
# Error: Invalid metric(s): vews
# Did you mean: 'vews' -> 'views'

# Check available metrics and dimensions
get_available_metrics()
get_available_metrics("view")  # Filter by pattern

get_available_dimensions() 
get_available_dimensions("country|city")  # Geographic dimensions
```

### 4. Data Transformation and Analysis

#### Convert to Data Frames
```{r dataframes}
# Get data and convert to clean data.frame
report <- get_daily_performance("last_30_days")
df <- yt_to_dataframe(report)
head(df)

# Convert to tibble (if tibble package installed)
tbl <- yt_to_tibble(report)

# Keep original column names
df_orig <- yt_to_dataframe(report, clean_names = FALSE)
```

#### Data Export
```{r export}
# Export to CSV
report <- get_top_videos("last_7_days")
file_path <- yt_export_csv(report, "top_videos.csv")

# Auto-generated filename with timestamp
file_path <- yt_export_csv(report)
```

#### Quick Visualization
```{r plots}
# Automatic plot creation (requires ggplot2)
daily_report <- get_daily_performance("last_30_days")
yt_quick_plot(daily_report)  # Line plot over time

# Bar chart for top videos
top_videos <- get_top_videos("last_7_days")
yt_quick_plot(top_videos, chart_type = "bar")

# Custom x and y columns
geo_data <- get_geographic_performance("last_30_days")
yt_quick_plot(geo_data, x_col = "country", y_col = "views")
```

#### Data Summary
```{r summary}
# Get summary statistics
report <- get_channel_overview("last_30_days")
summary <- yt_extract_summary(report)
print(summary)
```

### 5. Improved Error Handling and Diagnostics

#### Diagnostic Tools
```{r diagnostics}
# Check if everything is set up correctly
diagnose_tubern()

# Check API quota status
check_api_quota()
```

#### Error Recovery
```{r errors}
# tubern now provides helpful error messages and suggestions
# for authentication, API quota, parameter errors, etc.

# Example: If OAuth token expires, tubern will suggest running yt_oauth() again
# Example: If API quota is exceeded, tubern will suggest ways to reduce usage
```

## Complete Workflow Examples

### Example 1: Channel Performance Dashboard
```{r workflow1}
# Set up authentication
yt_oauth("your-client-id", "your-client-secret")

# Get overview data
overview <- get_channel_overview("last_30_days")
overview_df <- yt_to_dataframe(overview)

# Get daily trends
daily <- get_daily_performance("last_30_days")
daily_df <- yt_to_dataframe(daily)

# Get top videos
top_videos <- get_top_videos("last_30_days", max_results = 10)
videos_df <- yt_to_dataframe(top_videos)

# Get geographic performance
geo <- get_geographic_performance("last_30_days")
geo_df <- yt_to_dataframe(geo)

# Create visualizations
yt_quick_plot(daily)      # Daily trend
yt_quick_plot(top_videos) # Top videos
yt_quick_plot(geo)        # Geographic performance

# Export data
yt_export_csv(overview, "channel_overview.csv")
yt_export_csv(daily, "daily_performance.csv")
yt_export_csv(top_videos, "top_videos.csv")
```

### Example 2: Audience Analysis
```{r workflow2}
# Demographic analysis
demographics <- get_audience_demographics("last_90_days")
demo_df <- yt_to_dataframe(demographics)

# Device analysis  
devices <- get_report(
  ids = "channel==MINE",
  metrics = "views,estimatedMinutesWatched",
  dimensions = "deviceType",
  start_date = "last_90_days"
)
device_df <- yt_to_dataframe(devices)

# Geographic analysis
geography <- get_geographic_performance("last_90_days", max_results = 50)
geo_df <- yt_to_dataframe(geography)

# Combined analysis with dplyr (if available)
if (require(dplyr)) {
  # Top countries by watch time
  top_countries <- geo_df %>%
    arrange(desc(estimated_minutes_watched)) %>%
    head(10)
  
  print(top_countries)
}
```

### Example 3: Content Performance Analysis
```{r workflow3}
# Analyze video performance over time
videos_monthly <- get_top_videos("this_month", max_results = 50)
videos_df <- yt_to_dataframe(videos_monthly)

# Compare with previous month
videos_prev <- get_top_videos("last_month", max_results = 50) 
videos_prev_df <- yt_to_dataframe(videos_prev)

# Daily performance for trend analysis
daily_trends <- get_daily_performance(
  "last_30_days",
  metrics = c("views", "estimatedMinutesWatched", "likes", "comments")
)
trends_df <- yt_to_dataframe(daily_trends)

# Visualize trends
yt_quick_plot(daily_trends, x_col = "day", y_col = "views")
```

## Best Practices

### 1. Authentication
- Use `scope = "monetary"` only when you need revenue data
- Keep your OAuth credentials secure
- Use `diagnose_tubern()` to troubleshoot authentication issues

### 2. Data Retrieval
- Use relative dates for convenience: `"last_30_days"`, `"this_month"`
- Start with helper functions like `get_channel_overview()` before using `get_report()` directly
- Use `max_results` parameter to limit data when testing

### 3. Error Handling
- Always check `diagnose_tubern()` if you encounter issues
- Use `check_api_quota()` to monitor your API usage
- Enable `setup_guide = TRUE` for first-time setup help

### 4. Data Analysis
- Convert API responses to data.frames with `yt_to_dataframe()`
- Use `yt_quick_plot()` for rapid data exploration
- Export data with `yt_export_csv()` for external analysis

## Troubleshooting

### Common Issues

1. **Authentication Problems**
   ```{r}
   # Run diagnostics
   diagnose_tubern()
   
   # Re-authenticate
   yt_oauth("your-client-id", "your-client-secret")
   ```

2. **API Quota Exceeded**
   ```{r}
   # Check quota status
   check_api_quota()
   
   # Reduce data scope
   get_report(ids = "channel==MINE", metrics = "views", 
             start_date = "yesterday", max_results = 10)
   ```

3. **Invalid Parameters**
   ```{r}
   # Check available options
   get_available_metrics()
   get_available_dimensions()
   
   # Use helper functions for validation
   get_channel_overview("last_7_days")  # Pre-validated parameters
   ```

## More Resources

- [YouTube Analytics API Documentation](https://developers.google.com/youtube/analytics)
- [Google Cloud Console](https://console.cloud.google.com) for API setup
- [tubern GitHub Repository](https://github.com/gojiplus/tubern) for issues and updates

---
title: "Using tubern"
author: "Gaurav Sood"
date: "`r Sys.Date()`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Using tubern}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

## tubern: Access YouTube Analytics API via R


### Install, Load the package

To install the latest version from CRAN: 

```{r, eval=FALSE, install}
install.packages("tubern")
```

The latest development version of the package will always be on GitHub. Instructions for installing the package from Github are provided below.

```{r, eval=FALSE, install_g}
# install.packages('devtools')
devtools::install_github("soodoku/tubern", build_vignettes = TRUE)
```

Next, load the package: 

```{r, eval=FALSE, loadlib}
library(tubern)
```

### Using the package

To get going, get the application id and password from Google Developer Console (see [https://developers.google.com/youtube/reporting/](https://developers.google.com/youtube/reporting/)). Enable YouTube APIs. Then set the application id and password via the `yt_oauth` function. For more information about YouTube OAuth, see [YouTube OAuth Guide](https://developers.google.com/youtube/v3/guides/authentication).


```{r, eval=FALSE, auth}
yt_oauth("998136489867-5t3tq1g7hbovoj46dreqd6k5kd35ctjn.apps.googleusercontent.com", "MbOSt6cQhhFkwETXKur-L9rN")
```

```{r, eval=FALSE, prints}
## Waiting for authentication in browser...
## Press Esc/Ctrl + C to abort
## Authentication complete.
```

#### Get Statistics of a Video


```{r, eval=FALSE, getstats}
get_report(ids = "channel==MINE", metrics = "views", start_date = "2010-04-01", end_date ="2017-04-01")
```


```
## $kind
## [1] "youtubeAnalytics#resultTable"

## $columnHeaders
## $columnHeaders[[1]]
## $columnHeaders[[1]]$name
## [1] "views"

## $columnHeaders[[1]]$columnType
## [1] "METRIC"

## $columnHeaders[[1]]$dataType
## [1] "INTEGER"



## $rows
## $rows[[1]]
## $rows[[1]][[1]]
## [1] 6

```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/delete_group.R
\name{delete_group}
\alias{delete_group}
\title{Delete Group}
\usage{
delete_group(id, ...)
}
\arguments{
\item{id}{String. Required.
``The id parameter specifies the YouTube group ID of the group that is being deleted.''}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_DELETE}}.}
}
\value{
named list
}
\description{
Delete Group
}
\examples{

\dontrun{
delete_group(id="ABZZzGSIAAA")
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groups/delete}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tubern.R
\name{tubern_GET}
\alias{tubern_GET}
\title{Base POST AND GET functions. Not exported.}
\usage{
tubern_GET(path, query = NULL, ...)
}
\arguments{
\item{path}{path to specific API request URL}

\item{query}{query list}

\item{\dots}{Additional arguments passed to \code{\link[httr]{GET}}.}
}
\value{
list
}
\description{
GET
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation_helpers.R
\name{get_available_metrics}
\alias{get_available_metrics}
\title{Get available metrics with descriptions}
\usage{
get_available_metrics(pattern = NULL)
}
\arguments{
\item{pattern}{Optional regex pattern to filter metrics}
}
\value{
Named character vector of metrics and descriptions
}
\description{
Get available metrics with descriptions
}
\examples{
# Get all metrics
get_available_metrics()

# Get only view-related metrics
get_available_metrics("view")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_groups.R
\name{add_groups}
\alias{add_groups}
\title{Add Groups}
\usage{
add_groups(resource_details, ...)
}
\arguments{
\item{resource_details}{Named nested list. Required. Must provide:
snippet with title (required), and optionally description and contentDetails with itemType}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_POST}}.}
}
\value{
named list
}
\description{
Creates a new channel group.
}
\examples{

\dontrun{
add_groups(list(
  snippet = list(
    title = "My Channel Group",
    description = "A group for organizing channels"
  ),
  contentDetails = list(itemType = "youtube#channel")
))
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groups/insert}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{yt_extract_summary}
\alias{yt_extract_summary}
\title{Extract summary statistics from API response}
\usage{
yt_extract_summary(api_response)
}
\arguments{
\item{api_response}{API response from YouTube Analytics}
}
\value{
Named list with summary statistics
}
\description{
Extract summary statistics from API response
}
\examples{
\dontrun{
report <- get_channel_overview("last_30_days")
summary <- yt_extract_summary(report)
print(summary)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tubern.R
\name{yt_check_token}
\alias{yt_check_token}
\title{Check if authentication token is in options}
\usage{
yt_check_token()
}
\description{
Check if authentication token is in options
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_handling.R
\name{diagnose_tubern}
\alias{diagnose_tubern}
\title{Diagnose common tubern issues}
\usage{
diagnose_tubern()
}
\value{
Diagnostic information about tubern setup
}
\description{
Diagnose common tubern issues
}
\examples{
\dontrun{
diagnose_tubern()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{yt_export_csv}
\alias{yt_export_csv}
\title{Export data to CSV}
\usage{
yt_export_csv(api_response, filename = NULL, ...)
}
\arguments{
\item{api_response}{API response from YouTube Analytics}

\item{filename}{Output filename (default: auto-generated based on timestamp)}

\item{...}{Additional arguments passed to yt_to_dataframe()}
}
\value{
Path to saved file
}
\description{
Export data to CSV
}
\examples{
\dontrun{
report <- get_daily_performance("last_30_days")
file_path <- yt_export_csv(report, "daily_performance.csv")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{report_helpers}
\alias{report_helpers}
\title{Common Report Helper Functions}
\description{
Pre-configured functions for common YouTube Analytics report types
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation_helpers.R
\name{validation_helpers}
\alias{validation_helpers}
\title{YouTube Analytics API Validation Helpers}
\description{
Internal validation functions for metrics, dimensions, and parameters
using checkmate for robust parameter validation.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_group_items.R
\name{list_group_items}
\alias{list_group_items}
\title{List Group Items}
\usage{
list_group_items(group_id, ...)
}
\arguments{
\item{group_id}{String. Required. ID of the group}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_GET}}.}
}
\value{
named list
}
\description{
List Group Items
}
\examples{
\dontrun{
list_group_items(group_id = "vponEBg8hrR1yBUX0Hz66Uc5WMk/vyGp6PvFo4RvsFtPoIWeCReyIC8")
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groupItems/list}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_audience_demographics}
\alias{get_audience_demographics}
\title{Get audience demographics report}
\usage{
get_audience_demographics(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  dimension = c("ageGroup", "gender"),
  metrics = c("views", "estimatedMinutesWatched"),
  ...
)
}
\arguments{
\item{date_range}{Date range string or start date}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{dimension}{Demographic dimension: "ageGroup", "gender", or both (default: both)}

\item{metrics}{Vector of metrics to include (default: views, estimatedMinutesWatched)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with demographic data
}
\description{
Retrieves demographic breakdown of your audience by age and gender.
}
\examples{
\dontrun{
# Full demographic breakdown
get_audience_demographics("last_90_days")

# Age groups only
get_audience_demographics("this_month", dimension = "ageGroup")

# Gender breakdown only  
get_audience_demographics("this_quarter", dimension = "gender")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/update_group.R
\name{update_group}
\alias{update_group}
\title{Update Groups}
\usage{
update_group(resource_details, ...)
}
\arguments{
\item{resource_details}{Named nested list. Required. Must provide:
id, snippet title}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_PUT}}.}
}
\value{
named list
}
\description{
Update Groups
}
\examples{

\dontrun{
update_group(list(id="ABZZzGSIAAA", snippet = list(title ="hello")))
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groups/update}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_helpers.R
\name{resolve_date_range}
\alias{resolve_date_range}
\title{Resolve date range with support for relative dates}
\usage{
resolve_date_range(start_date, end_date = NULL)
}
\arguments{
\item{start_date}{Start date (can be relative like "last_30_days" or absolute like "2024-01-01")}

\item{end_date}{End date (can be relative or absolute). If NULL and start_date is relative, will be calculated automatically}
}
\value{
List with resolved start_date and end_date as YYYY-MM-DD strings
}
\description{
Resolve date range with support for relative dates
}
\examples{
# Relative date ranges
resolve_date_range("last_30_days")
resolve_date_range("this_month")
resolve_date_range("last_quarter")

# Absolute date ranges  
resolve_date_range("2024-01-01", "2024-01-31")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/list_groups.R
\name{list_groups}
\alias{list_groups}
\title{List Groups}
\usage{
list_groups(filter, page_token = NULL, ...)
}
\arguments{
\item{filter}{Named Vector. Required. Only one of the two: id or mine.
id: Comma-separated list of YouTube group ID(s) to retrieve.
mine: Set to TRUE to retrieve all groups owned by the authenticated user.}

\item{page_token}{String. Optional. Identifies a specific page in the result set that should be returned.}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_GET}}.}
}
\value{
named list
}
\description{
List Groups
}
\examples{
\dontrun{
list_groups(filter = c(mine = TRUE))
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groups/list}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_handling.R
\name{error_handling}
\alias{error_handling}
\title{Enhanced Error Handling for tubern}
\description{
Custom error classes and improved error handling for YouTube Analytics API
using rlang for modern error handling patterns.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_daily_performance}
\alias{get_daily_performance}
\title{Get daily performance time series}
\usage{
get_daily_performance(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  metrics = c("views", "estimatedMinutesWatched"),
  ...
)
}
\arguments{
\item{date_range}{Date range string or start date}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{metrics}{Vector of metrics to include (default: views, estimatedMinutesWatched)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with daily time series data
}
\description{
Retrieves day-by-day performance metrics for trend analysis.
}
\examples{
\dontrun{
# Daily views for last 30 days
get_daily_performance("last_30_days")

# Daily performance with engagement metrics
get_daily_performance("this_month", metrics = c("views", "likes", "comments", "shares"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_helpers.R
\name{date_helpers}
\alias{date_helpers}
\title{Date Helper Functions for tubern}
\description{
Convenience functions for handling dates in YouTube Analytics API requests
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_geographic_performance}
\alias{get_geographic_performance}
\title{Get geographic performance report}
\usage{
get_geographic_performance(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  dimension = "country",
  metrics = c("views", "estimatedMinutesWatched"),
  max_results = 25,
  ...
)
}
\arguments{
\item{date_range}{Date range string or start date}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{dimension}{Geographic dimension: "country", "province", or "city" (default: "country")}

\item{metrics}{Vector of metrics to include (default: views, estimatedMinutesWatched)}

\item{max_results}{Number of results to return (default: 25)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with geographic data
}
\description{
Retrieves performance metrics broken down by country or other geographic dimensions.
}
\examples{
\dontrun{
# Top countries by views
get_geographic_performance("last_30_days")

# US states/provinces (requires US-only data)
get_geographic_performance("this_month", dimension = "province", 
                          filters = "country==US")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tubern.R
\name{tubern_POST}
\alias{tubern_POST}
\title{POST}
\usage{
tubern_POST(path, query = NULL, body = "", ...)
}
\arguments{
\item{path}{path to specific API request URL}

\item{query}{query list}

\item{body}{passing image through body}

\item{\dots}{Additional arguments passed to \code{\link[httr]{POST}}.}
}
\value{
list
}
\description{
POST
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/validation_helpers.R
\name{get_available_dimensions}
\alias{get_available_dimensions}
\title{Get available dimensions with descriptions}
\usage{
get_available_dimensions(pattern = NULL)
}
\arguments{
\item{pattern}{Optional regex pattern to filter dimensions}
}
\value{
Named character vector of dimensions and descriptions
}
\description{
Get available dimensions with descriptions
}
\examples{
# Get all dimensions
get_available_dimensions()

# Get only geographic dimensions
get_available_dimensions("country|city")
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tubern.R
\docType{package}
\name{tubern}
\alias{tubern-package}
\alias{tubern}
\title{\pkg{tubern} provides access to the YouTube Analytics and Reporting API}
\description{
Provides access to YouTube Analytics API v2 for retrieving YouTube Analytics data including views, engagement metrics, demographics, and revenue data. Supports OAuth 2.0 authentication and channel group management. See \url{https://developers.google.com/youtube/analytics/} for API documentation.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/gojiplus/tubern}
  \item Report bugs at \url{https://github.com/gojiplus/tubern/issues}
}

}
\author{
\strong{Maintainer}: Gaurav Sood \email{gsood07@gmail.com}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{yt_to_tibble}
\alias{yt_to_tibble}
\title{Convert API response to tibble (if tibble is available)}
\usage{
yt_to_tibble(api_response, ...)
}
\arguments{
\item{api_response}{API response from YouTube Analytics}

\item{...}{Additional arguments passed to yt_to_dataframe()}
}
\value{
tibble or data.frame if tibble not available
}
\description{
Convert API response to tibble (if tibble is available)
}
\examples{
\dontrun{
report <- get_top_videos("last_7_days")
tbl <- yt_to_tibble(report)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/add_group_item.R
\name{add_group_item}
\alias{add_group_item}
\title{Add Group Item}
\usage{
add_group_item(resource_details, ...)
}
\arguments{
\item{resource_details}{Named nested list. Required. Must provide:
groupId, resource.id}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_POST}}.}
}
\value{
named list
}
\description{
Add Group Item
}
\examples{

\dontrun{
add_group_item(list(groupId = "", resource.id ="hello"))
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/groupItems/insert}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/date_helpers.R
\name{get_common_date_ranges}
\alias{get_common_date_ranges}
\title{Get common date ranges}
\usage{
get_common_date_ranges()
}
\value{
Named list of common date ranges
}
\description{
Get common date ranges
}
\examples{
get_common_date_ranges()
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_top_videos}
\alias{get_top_videos}
\title{Get top videos report}
\usage{
get_top_videos(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  max_results = 10,
  metrics = c("views", "likes", "comments"),
  ...
)
}
\arguments{
\item{date_range}{Date range string or start date}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{max_results}{Number of top videos to return (default: 10)}

\item{metrics}{Vector of metrics to include (default: views, likes, comments)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with top videos data
}
\description{
Retrieves performance metrics for individual videos, sorted by views.
}
\examples{
\dontrun{
# Top 10 videos by views in last 30 days
get_top_videos("last_30_days")

# Top 25 videos with more metrics
get_top_videos("this_month", max_results = 25, 
               metrics = c("views", "likes", "comments", "shares", "estimatedMinutesWatched"))
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/error_handling.R
\name{check_api_quota}
\alias{check_api_quota}
\title{Check API quota status and provide guidance}
\usage{
check_api_quota()
}
\value{
Message about current quota usage (if available)
}
\description{
Check API quota status and provide guidance
}
\examples{
\dontrun{
check_api_quota()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_revenue_report}
\alias{get_revenue_report}
\title{Get revenue report (requires monetary scope)}
\usage{
get_revenue_report(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  currency = "USD",
  include_cpm = TRUE,
  ...
)
}
\arguments{
\item{date_range}{Date range string or start date}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{currency}{Currency code (default: "USD")}

\item{include_cpm}{Logical. Include CPM metrics (default: TRUE)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with revenue data
}
\description{
Retrieves revenue and monetization metrics.
}
\examples{
\dontrun{
# Revenue report for last month
get_revenue_report("last_month")

# Revenue in different currency
get_revenue_report("this_quarter", currency = "EUR")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/yt_oauth.R
\name{yt_oauth}
\alias{yt_oauth}
\title{Set up Authorization}
\usage{
yt_oauth(
  app_id = NULL,
  app_secret = NULL,
  scope = "analytics",
  token = ".httr-oauth",
  setup_guide = interactive(),
  ...
)
}
\arguments{
\item{app_id}{Client ID from Google Cloud Console; required; no default}

\item{app_secret}{Client secret from Google Cloud Console; required; no default}

\item{scope}{Character. One of:
\itemize{
  \item \code{"analytics"} - Basic analytics data (views, likes, etc.) - Default
  \item \code{"monetary"} - Includes revenue and monetization data
  \item \code{"auto"} - Automatically detect scope based on first API call
}}

\item{token}{Path to file containing the token. Default is \code{.httr-oauth} in working directory.}

\item{setup_guide}{Logical. Show setup guide for first-time users (default: TRUE for interactive sessions)}

\item{\dots}{Additional arguments passed to \code{\link[httr]{oauth2.0_token}}}
}
\value{
Sets the google_token option and saves .httr-oauth in working directory
}
\description{
Simplified OAuth2 setup for YouTube Analytics API. This function will automatically
detect the required scope based on your needs and provide helpful setup guidance.
}
\details{
The function looks for .httr-oauth in the working directory. If it doesn't find it,
it expects an application ID and a secret. The function launches a browser to allow
you to authorize the application.
}
\examples{
 \dontrun{
   # Basic setup (will show setup guide first time)
   yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret")

   # Setup with monetary scope for revenue data
   yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret",
            scope = "monetary")

   # Skip setup guide
   yt_oauth("your-client-id.apps.googleusercontent.com", "your-client-secret",
            setup_guide = FALSE)
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{yt_quick_plot}
\alias{yt_quick_plot}
\title{Create a quick visualization of the data (if ggplot2 is available)}
\usage{
yt_quick_plot(api_response, x_col = NULL, y_col = NULL, chart_type = "auto")
}
\arguments{
\item{api_response}{API response from YouTube Analytics}

\item{x_col}{Column name for x-axis (auto-detected if NULL)}

\item{y_col}{Column name for y-axis (auto-detected if NULL)}

\item{chart_type}{Type of chart: "line", "bar", "point" (default: auto)}
}
\value{
ggplot object or base R plot if ggplot2 not available
}
\description{
Create a quick visualization of the data (if ggplot2 is available)
}
\examples{
\dontrun{
# Daily views over time
daily_report <- get_daily_performance("last_30_days")
yt_quick_plot(daily_report)

# Top videos by views
top_videos <- get_top_videos("last_7_days")
yt_quick_plot(top_videos, chart_type = "bar")
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{yt_to_dataframe}
\alias{yt_to_dataframe}
\title{Convert API response to data frame}
\usage{
yt_to_dataframe(api_response, clean_names = TRUE, parse_dates = TRUE)
}
\arguments{
\item{api_response}{List returned from get_report() or other API functions}

\item{clean_names}{Logical. Clean column names by removing special characters (default: TRUE)}

\item{parse_dates}{Logical. Parse date columns to Date objects (default: TRUE)}
}
\value{
data.frame with transformed data, or NULL if no data available
}
\description{
Transforms YouTube Analytics API response into a clean data.frame with proper column names and types.
}
\examples{
\dontrun{
# Get data and convert to data.frame
report <- get_channel_overview("last_30_days")
df <- yt_to_dataframe(report)

# Keep original column names
df <- yt_to_dataframe(report, clean_names = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/report_helpers.R
\name{get_channel_overview}
\alias{get_channel_overview}
\title{Get channel overview report}
\usage{
get_channel_overview(
  date_range,
  end_date = NULL,
  ids = "channel==MINE",
  include_engagement = TRUE,
  include_subscribers = TRUE,
  ...
)
}
\arguments{
\item{date_range}{Date range string like "last_30_days" or start date for custom range}

\item{end_date}{End date (only needed if date_range is a specific start date)}

\item{ids}{Channel identifier (default: "channel==MINE")}

\item{include_engagement}{Logical. Include likes, dislikes, comments, shares (default: TRUE)}

\item{include_subscribers}{Logical. Include subscriber metrics (default: TRUE)}

\item{...}{Additional arguments passed to get_report()}
}
\value{
API response with channel overview data
}
\description{
Retrieves key channel performance metrics for a specified time period.
}
\examples{
\dontrun{
# Channel performance for last 30 days
get_channel_overview("last_30_days")

# Channel performance for specific date range
get_channel_overview("2024-01-01", end_date = "2024-01-31")

# Basic metrics only
get_channel_overview("this_month", include_engagement = FALSE, include_subscribers = FALSE)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_report.R
\name{get_report}
\alias{get_report}
\title{Get Reports}
\usage{
get_report(
  ids,
  metrics,
  start_date = NULL,
  end_date = NULL,
  currency = NULL,
  dimensions = NULL,
  filters = NULL,
  include_historical_channel_data = NULL,
  max_results = NULL,
  sort = NULL,
  start_index = NULL,
  ...
)
}
\arguments{
\item{ids}{String. Channel or content owner identifier.
For channels: \code{"channel==MINE"} (for authenticated user's channel) or
\code{"channel==UCxxxxxxxxxxxxxxx"} (for specific channel ID).
For content owners: \code{"contentOwner==ownerName"}.}

\item{metrics}{String. Comma-separated list of YouTube Analytics metrics,
such as \code{views} or \code{likes,dislikes}.}

\item{start_date}{String. Start date for the report.
Can be in YYYY-MM-DD format or relative date like "last_30_days", "this_month", "yesterday".}

\item{end_date}{String. End date for the report.
Can be in YYYY-MM-DD format or relative date. If NULL and start_date is relative, will be calculated automatically.}

\item{currency}{Optional. String. Default is USD. Specifies what earnings
metrics like \code{earnings, adEarnings, grossRevenue, playbackBasedCpm,
impressionBasedCpm} will be reported in.}

\item{dimensions}{String. Optional. Comma-separated list of YouTube Analytics
dimensions, such as \code{video} or \code{ageGroup,gender}.}

\item{filters}{String. Optional. Dimension value filters. Multiple filters
can be separated by semicolons. For video filtering:
\code{"video==videoId1,videoId2"} (comma-separated video IDs).
For country filtering: \code{"country==US"}.
Combined example: \code{"video==videoId1,videoId2;country==US"}.
Note: When filtering by video IDs, ensure the \code{dimensions} parameter
includes "video".}

\item{include_historical_channel_data}{Boolean. Default is FALSE.
``Whether the API response should include channels' watch time and view data
from the time period prior to when the channels were linked to the content
owner.''}

\item{max_results}{Integer. Optional. The maximum number of rows to
include in the response.}

\item{sort}{String. Optional A comma-separated list of dimensions or metrics
that determine the sort order for YouTube}

\item{start_index}{Integer. Optional. ``The 1-based index of the first
entity to retrieve.''}

\item{\dots}{Additional arguments passed to \code{\link[tubern]{tubern_GET}}.}
}
\value{
named list
}
\description{
Retrieves YouTube Analytics reports containing YouTube Analytics data.
}
\section{Troubleshooting 404 Errors}{

If you encounter a 404 "Not Found" error, check the following:
\itemize{
  \item Ensure YouTube Analytics API is enabled in your Google Cloud Console
  project
  \item Verify your authentication token is valid and has the correct scopes
  \item Check that the channel ID (if using specific channel ID) exists and
  you have access to it
  \item Use \code{"channel==MINE"} to access your own authenticated channel's
  data
}
}

\examples{
\dontrun{
# Basic channel report
get_report(ids = "channel==MINE", metrics = "views",
           start_date = "2020-01-01", end_date = "2020-01-31")

# Report with video filtering (requires dimensions = "video")
get_report(ids = "channel==MINE",
           metrics = "views,likes,comments",
           dimensions = "video",
           filters = "video==videoId1,videoId2",
           start_date = "2020-01-01", end_date = "2020-01-31")

# Report with country filtering
get_report(ids = "channel==MINE",
           metrics = "views",
           filters = "country==US",
           start_date = "2020-01-01", end_date = "2020-01-31")
}
}
\references{
\url{https://developers.google.com/youtube/analytics/reference/reports/query}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_transformation.R
\name{data_transformation}
\alias{data_transformation}
\title{Data Transformation Utilities}
\description{
Functions to transform YouTube Analytics API responses into common R data formats
}
